@@ -26,8 +26,9 @@ from graphlab.connect import _get_metric_tracker
 
 CONFIG_SECTION = 'AWS'
 DEFAULT_CIDR_RULE = '0.0.0.0/0'  # Open to the entire internet.
+DEFAULT_INSTANCE_TYPE = 'm3.xlarge'
 GRAPHLAB_PORT_MIN_NUM = 9000
-GRAPHLAB_PORT_MAX_NUM = 9003
+GRAPHLAB_PORT_MAX_NUM = 9004
 JSON_BLOB_PATH_FORMAT = "/gl_api/api/v1/cloud/aws/instances/%s?client_version=%s&region=%s"
 GRAPHLAB_NAME = 'GraphLab'
 
@@ -38,11 +39,10 @@ __LOGGER__ = logging.getLogger(__name__)
 
 class _EC2(glserver.RemoteServer):
     """
-    The class which manages the connection information for a unity server
-    running on an EC2 instance.
+    This class is responsible for starting up an EC2 instance.
     """
     def __init__(self, instance_type, region, CIDR_rule, auth_token, product_key, security_group_name, 
-                 tags, public_key, secret_key):
+                 tags, public_key, secret_key, user_data, credentials):
         self.logger = __LOGGER__
         self.region = region
         self.auth_token = auth_token
@@ -51,7 +51,7 @@ class _EC2(glserver.RemoteServer):
         _get_metric_tracker().track(('ec2.region.%s' % region), value=1)
 
         try:
-            conn = boto.ec2.connect_to_region(self.region)
+            conn = boto.ec2.connect_to_region(self.region, **credentials)
 
             # Does the security group already exist?
             security_group = None
@@ -80,7 +80,7 @@ class _EC2(glserver.RemoteServer):
             if not "<Code>InvalidPermission.Duplicate</Code>" in str(e):
                 raise
 
-        if (config.mode == 'UNIT' and 'GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ):
+        if (config.mode != 'PROD' and 'GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ):
             # unit-test mode, don't involve webservice to retrieve AMI, instead use environment variables
             ami_id = os.environ['GRAPHLAB_TEST_AMI_ID']
             engine_url = os.environ['GRAPHLAB_TEST_ENGINE_URL']
@@ -104,7 +104,7 @@ class _EC2(glserver.RemoteServer):
         self.logger.debug("web service return: %s" % json_blob)
 
         if json_blob['ami_id'] is None:
-            raise Exception("Unable to successfully retrieve correct EC2 image to launch. Please try again later or contact support@graphlab.com. Error received:'%s'", json_blob['message'])
+            raise Exception("Unable to successfully retrieve correct EC2 image to launch. Please try again later or contact support@graphlab.com. Error received:'%s'" % json_blob['message'])
         ami_id = json_blob['ami_id']
 
         if self.auth_token is not None:
@@ -120,55 +120,60 @@ class _EC2(glserver.RemoteServer):
             product_key = ''
         json_blob['product_key'] = product_key
 
+        user_data.update(json_blob)
+
         run_instances_args =  {
             'security_group_ids' : [ security_group_name ],
-            'user_data' : json.dumps(json_blob),
+            'user_data' : json.dumps(user_data),
             'instance_type' : instance_type
         }
 
-        if (config.mode == 'UNIT' and 'GRAPHLAB_TEST_EC2_KEYPAIR' in os.environ):
+        if (config.mode != 'PROD' and 'GRAPHLAB_TEST_EC2_KEYPAIR' in os.environ):
             keypair = os.environ['GRAPHLAB_TEST_EC2_KEYPAIR']
             self.logger.info("UNIT mode, using keypair: '%s' when launching EC2 instance" % (keypair))
             run_instances_args['key_name'] = keypair
 
-        # Launch the EC2 instance
+        # Launch the EC instance
         response = conn.run_instances(ami_id, **run_instances_args)
 
         if(not(len(response.instances) == 1)):
             raise Exception('Could not successfully start EC2 instance.')
 
-        instance = response.instances[0]
+        self.instance = response.instances[0]
 
         # If auth_token is not set by the input argument, the EC2 instance
         # will use its instance id as the default auth_token, so we reset
         # self.auth_token after the instance is running.
         if not self.auth_token:
-            self.auth_token = instance.id
+            self.auth_token = self.instance.id
 
-        self.logger.info("Launching an %s instance in the %s availability zone, with id: %s. You will be responsible for the cost of this instance." % (instance.instance_type, instance.placement, instance.id))
+        self.logger.info("Launching an %s instance in the %s availability zone, with id: %s. You will be responsible for the cost of this instance." % (self.instance.instance_type, self.instance.placement, self.instance.id))
 
         # Wait for the host to start up
-        while not instance.update() == 'pending':
+        while not self.instance.update() == 'pending':
             time.sleep(1)
-        while instance.update() == 'pending':
+        while self.instance.update() == 'pending':
             time.sleep(1)
 
-        if instance.state == 'running':
-            self.instance_id = instance.id
-            server_addr = 'tcp://' + instance.ip_address + ':' + str(GRAPHLAB_PORT_MIN_NUM)
+        if self.instance.state == 'running':
+            self.instance_id = self.instance.id
+            server_addr = 'tcp://' + self.instance.ip_address + ':' + str(GRAPHLAB_PORT_MIN_NUM)
             self.logger.debug("EC2 instance is now running, attempting to connect to engine.")
             super(self.__class__, self).__init__(server_addr, auth_token=self.auth_token, 
                                                  public_key=public_key)
         else:   # Error case
             self.logger.error('Could not startup EC2 instance.')
-            if(hasattr(instance, 'id')):
+            if(hasattr(self.instance, 'id')):
                 self.stop()
+            return 
+        
+        self.public_dns_name = self.instance.public_dns_name
 
         # Add tags to this instance.
         if(tags is None):
             tags = {}
         tags[GRAPHLAB_NAME] = ''
-        conn.create_tags(instance.id, tags)
+        conn.create_tags(self.instance.id, tags)
 
 
     def get_auth_token(self):
@@ -217,7 +222,7 @@ def get_credentials():
     return (os.environ['AWS_ACCESS_KEY_ID'], os.environ['AWS_SECRET_ACCESS_KEY'])
 
 
-def launch_EC2(instance_type='m3.xlarge', region=None, CIDR_rule=None, auth_token=None,
+def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None, auth_token=None,
                security_group=None, tags=None, use_secure_connection=True):
     """
     Launches an EC2 instance of the specified type. It is expected that AWS credentials are already set
@@ -227,8 +232,8 @@ def launch_EC2(instance_type='m3.xlarge', region=None, CIDR_rule=None, auth_toke
     ----------
     instance_type : string, optional
         The EC2 instance type to launch, default is m3.xlarge. We support all instance types
-        except for 'm1.small' and 't1.micro'. For a list of instance_types, please
-        refer to `here <http://aws.amazon.com/ec2/instance-types/#Instance_Types>`_.
+        except: 't2.micro', 't2.small', 't2.medium' and 'm3.medium'. For a list of instance_types,
+        please refer to `here <http://aws.amazon.com/ec2/instance-types/#Instance_Types>`_.
 
     region : string, optional
         The AWS region in which to launch your instance. Default is 'us-west-2'.
@@ -261,50 +266,15 @@ def launch_EC2(instance_type='m3.xlarge', region=None, CIDR_rule=None, auth_toke
 
     assert not glconnect.is_connected()
 
-    # get and validate the product / registration key
-    try:
-        product_key = graphlab.product_key.get_product_key()
-    except KeyError as k:
-        __LOGGER__.error(k.message)
-        return
-
-    if (not graphlab.product_key.is_product_key_valid(product_key)):
-        __LOGGER__.error("Product Key validation failed, please confirm your product key is correct. "
-                         "If you believe this key to be valid, please contact support@graphlab.com")
-        return
-
-    # Determine region
-    if(region is None):
-        region = _get_region_from_config()
-        if(region is None):
-            region = 'us-west-2'
-
-    # If a CIDR rule is specified, a security group must also be specified.
-    if CIDR_rule is not None and security_group is None:
-        __LOGGER__.error('If you specify a CIDR rule, you must also specify an existing security group.')
-        return
-    # If no security group is specified (and no CIDR rule), use a default value for the security group name.
-    elif security_group is None:
-        security_group = GRAPHLAB_NAME
-
-    if CIDR_rule is None:
-        CIDR_rule = DEFAULT_CIDR_RULE
-
     (server_public_key, server_secret_key, client_public_key, client_secret_key) = ("", "", "", "")
     if use_secure_connection:
         (server_public_key, server_secret_key) = get_public_secret_key_pair()
         (client_public_key, client_secret_key) = get_public_secret_key_pair()
         __LOGGER__.debug('Launching server with public key: %s' % server_public_key)
 
-    # By default the EC2 instance will use its instance id as the auth_token.
-    try:
-        server = _EC2(instance_type, region, CIDR_rule, auth_token, product_key, security_group,
-                      tags, server_public_key, server_secret_key)    # Launches the EC2 instance
-    except Exception as e:
-      __LOGGER__.error("Unable to launch EC2 instance: '%s'. Please check AWS Console to make sure any EC2 instances launched have been terminated." % e)
-      # since EC2 launch could be part of a script, we want to raise the exception so that remaining code
-      # stops executing (otherwise a local server would be created and the code would continue locally)
-      raise e
+    server = _ec2_factory(instance_type=instance_type, region=region, CIDR_rule=CIDR_rule, auth_token=auth_token,
+                          security_group=security_group, tags=tags, server_public_key=server_public_key,
+                          server_secret_key=server_secret_key)
 
     try:
         # Once the EC2 instance starts up, the client still need to wait for graphlab engine to start up.
@@ -388,3 +358,49 @@ def terminate_EC2():
         glconnect.stop()
     else:
         __LOGGER__.info("No EC2 instance to stop.")
+
+
+def _ec2_factory(instance_type, region=None, CIDR_rule=None, auth_token=None, security_group=None,
+                 tags=None, server_public_key='', server_secret_key='', user_data = {},
+                 credentials = {}):
+
+    # get and validate the product / registration key
+    try:
+        product_key = graphlab.product_key.get_product_key()
+    except KeyError as k:
+        __LOGGER__.error(k.message)
+        return
+
+    if (not graphlab.product_key.is_product_key_valid(product_key)):
+        __LOGGER__.error("Product Key validation failed, please confirm your product key is correct. "
+                         "If you believe this key to be valid, please contact support@graphlab.com")
+        return
+
+    # Determine region
+    if(region is None):
+        region = _get_region_from_config()
+        if(region is None):
+            region = 'us-west-2'
+
+    # If a CIDR rule is specified, a security group must also be specified.
+    if CIDR_rule is not None and security_group is None:
+        __LOGGER__.error('If you specify a CIDR rule, you must also specify an existing security group.')
+        return
+    # If no security group is specified (and no CIDR rule), use a default value for the security group name.
+    elif security_group is None:
+        security_group = GRAPHLAB_NAME
+
+    if CIDR_rule is None:
+        CIDR_rule = DEFAULT_CIDR_RULE
+
+    # By default the EC2 instance will use its instance id as the auth_token.
+    try:
+        server = _EC2(instance_type, region, CIDR_rule, auth_token, product_key, security_group,
+                      tags, server_public_key, server_secret_key, user_data, credentials)    # Launches the EC2 instance
+    except Exception as e:
+      __LOGGER__.error("Unable to launch EC2 instance: '%s'. Please check AWS Console to make sure any EC2 instances launched have been terminated." % e)
+      # since EC2 launch could be part of a script, we want to raise the exception so that remaining code
+      # stops executing (otherwise a local server would be created and the code would continue locally)
+      raise e
+
+    return server
