@@ -12,10 +12,15 @@ import of the graphlab package.
 import graphlab.connect.main as glconnect
 from graphlab.data_structures.sframe import SFrame
 from graphlab.data_structures.sarray import SArray
+from graphlab.data_structures.gframe import GFrame, VERTEX_GFRAME, EDGE_GFRAME
 from graphlab.cython.cy_graph import UnityGraphProxy
 from graphlab.cython.context import debug_trace as cython_context
 from graphlab.util import make_internal_url
-import pandas as pd
+import graphlab.canvas
+import graphlab.canvas.inspect
+from graphlab.deps import pandas as pd
+from graphlab.deps import HAS_PANDAS
+import inspect
 import copy
 
 ## \internal Default column name for vertex id.
@@ -27,15 +32,6 @@ _SRC_VID_COLUMN = '__src_id'
 ## \internal Default column name for target vid.
 _DST_VID_COLUMN = '__dst_id'
 
-## \internal Flag indicates the existance of matplotlib and networkX.
-_HAS_MATPLOTLIB = False
-try:
-    import networkx as nx
-    import matplotlib.pyplot as plt
-    _HAS_MATPLOTLIB = True
-except:
-    pass
-
 
 #/**************************************************************************/
 #/*                                                                        */
@@ -140,18 +136,15 @@ class Edge(object):
 
 class SGraph(object):
     """
-    A directed graph that allows arbitrary dictionary attributes on vertices
-    and edges. SGraph uses :class:`~graphlab.SFrame` as its storage
-    backend and therefore can scale to very big data.
+    A scalable graph data structure backed by :class:`~graphlab.SFrame`
+    that allows arbitrary dictionary attributes on vertices
+    and edges, and provides flexible vertex/edge query functions,
+    and seamless transformation from(to) :class:`~graphlab.SFrame`.
 
-    The constructor creates an empty graph, to which vertices and edges
-    can be added. Alternatively, the :py:func:`load_graph` function creates a
-    :class:`~graphlab.SGraph` from a binary or text file.
+    **Construction**
 
-    There are several ways to create a
-    :class:`~graphlab.SGraph`. The simplest is to make an empty graph then add
-    vertices and edges. Note that SGraphs are immutable, so a new SGraph is
-    returned when vertices and edges are added.
+    There are several ways to create a :class:`~graphlab.SGraph`.
+    The simplest one is to make an empty graph then add vertices and edges.
 
     >>> import graphlab
     >>> from graphlab import SGraph, Vertex, Edge, SFrame
@@ -164,30 +157,42 @@ class SGraph(object):
     >>> g = g.add_vertices(verts)
     >>> g = g.add_edges(Edge(1, 2))
 
-    Because :class:`~graphlab.SGraph` is immutable we can chain these steps
-    together to make a new graph in a single line.
+    Note that SGraphs are *structually immutable*. In the above code,
+    add_vertices and add_edges both return a new graph; the old
+    graph gets garbage collected.
+
+    We can chain these steps together to make a new graph in a single line.
 
     >>> g = SGraph().add_vertices([Vertex(i) for i in range(10)]).add_edges(
             [Edge(i, i+1) for i in range(9)])
 
-    SGraphs can also be created from an edge list stored in an
+    SGraphs can also be created from just an edge list stored in an
     :class:`~graphlab.SFrame`. Vertices are added to the graph automatically
     based on the edge list, and columns of the SFrame not used as source or
     destination vertex IDs are assumed to be edge attributes. For this example
     we download a dataset of James Bond characters to an SFrame, then build the
     graph.
 
-    >>> data = SFrame.read_csv('http://s3.amazonaws.com/GraphLab-Datasets/bond/bond_edges.csv')
+    >>> edge_data = SFrame.read_csv('http://s3.amazonaws.com/GraphLab-Datasets/bond/bond_edges.csv')
+    >>> vertex_data = SFrame.read_csv('http://s3.amazonaws.com/GraphLab-Datasets/bond/bond_vertices.csv')
     >>> g = SGraph()
-    >>> g = g.add_edges(data, src_field='src', dst_field='dst')
+    >>> g = g.add_edges(edge_data, src_field='src', dst_field='dst')
+
+    The SGraph constructor can also take in a vertex SFrame and edge SFrame directly.
+    For example, we can construct the same James Bond graph using the following one-liner:
+
+    >>> g = SGraph(vertices=vertex_data, edges=edge_data, vid_field='name',
+                   src_field='src', dst_field='dst')
 
     Finally, a :class:`~graphlab.SGraph` can be created directly from a file,
-    either local or remote, using the :py:func:`load_graph` method. Loading a
+    either local or remote, using the :py:func:`load_sgraph` method. Loading a
     graph with this method works with both the native binary save format and a
     variety of text formats.
 
     >>> g.save('james_bond') # the graph will be saved to a directory named 'james_bond'.
-    >>> newgraph = graphlab.load_graph('james_bond')
+    >>> newgraph = graphlab.load_sgraph('james_bond')
+
+    **Inspecting SGraph**
 
     Small graphs can be explored very efficiently with the :py:func:`show`
     method, which displays a plot of the graph. The vertex labels can be IDs or
@@ -229,21 +234,114 @@ class SGraph(object):
     4  managed_by      Moneypenny              M
 
 
+    **Modifying SGraph**
+
+    SGraph is *structually immutable*, but the data stored on vertices and edges
+    can be mutated.
+
+    Modifying data on the SGraph can be done through 2 *special PROPERTIES*:
+    :py:attr:`~graphlab.SGraph.vertices` and :py:attr:`~graphlab.SGraph.edges`.
+
+    :py:attr:`~graphlab.SGraph.vertices` is a *special SFrame* object associated
+    with the vertex data of the SGraph, and similarly
+    :py:attr:`~graphlab.SGraph.edges` is a *special* SFrame associated with the edge
+    data. The following example show the difference between the special
+    graph-related SFrames and the normal SFrames.
+
+    For example:
+
+    >>> print g.edges.head(5)
+         relation        __src_id       __dst_id
+    0    worksfor               M              Q
+    1      friend  Inga Bergstorm     James Bond
+    2   killed_by   Elliot Carver     James Bond
+    3   engagedto    Paris Carver  Elliot Carver
+    4  managed_by      Moneypenny              M
+
+    This code gives the exact same answer as
+
+    >>> print g.get_edges().head(5)
+
+    However, the difference is that the return value of g.get_edges() is a normal
+    SFrame indepedent from g, whereas g.edges is bound to g.
+    We can modify the edge data using this special edge SFrame.
+
+    >>> # The following code MUTATES the relation attribute on the edges of g
+    >>> g.edges['relation'] = g.edges['relation'].apply(lambda x: x[0].upper())
+    >>> print g.get_edges().head(5)
+         relation        __src_id       __dst_id
+    0    W                      M              Q
+    1    F         Inga Bergstorm     James Bond
+    2    K          Elliot Carver     James Bond
+    3    E           Paris Carver  Elliot Carver
+    4    M             Moneypenny              M
+
+    >>> # The following code DOES NOT MUTATE the relation attribute on the edges of g
+    >>> e = g.get_edges()  # e is a normal SFrame independent of g.
+    >>> e['relation'] = e['relation'].apply(lambda x: x[0].lower())
+    >>> print g.get_edges().head(5)
+         relation        __src_id       __dst_id
+    0    W                      M              Q
+    1    F         Inga Bergstorm     James Bond
+    2    K          Elliot Carver     James Bond
+    3    E           Paris Carver  Elliot Carver
+    4    M             Moneypenny              M
+
+    Calling a method like head(), tail(), or append() on a graph-related SFrame would result
+    in a new instance of a regular SFrame. For example, the following code DOES NOT MUTATE g.
+
+    >>> e = g.edges.head(5)
+    >>> e['is_friend'] = e['relation'].apply(lambda x: x[0] == 'F')
+
+    Another important difference of these two special SFrames are that the '__id',
+    '__src_id', and '__dst_id' are not mutable because changing them will
+    change the structure of the graph and SGraph is *structually immutable*.
+
+    Otherwise, g.vertices and g.edges act like normal SFrames, and it makes
+    mutating graph data super easy. For example, initializing (removing) an edge field
+    is the same as adding (removing) a column to(from) an SFrame:
+
+    >>> g.edges['weight'] = 1.0
+    >>> del g.edges['weight']
+
     See Also
     --------
     Vertex, Edge, SFrame
     """
 
-    __slots__ = ['__proxy__']
+    __slots__ = ['__proxy__', '_vertices', '_edges']
 
-    def __init__(self, _proxy=None):
-        """__init__()
-        Construct an empty SGraph.
+    def __init__(self, vertices=None, edges=None, vid_field='__id',
+                 src_field='__src_id', dst_field='__dst_id', _proxy=None):
+        """
+        __init__(vertices=None, edges=None, vid_field='__id', src_field='__src_id', dst_field='__dst_id')
+
+        By default, construct an empty graph when vertices and edges are None.
+        Otherwise construct an SGraph with given vertices and edges.
+
+        Parameters
+        ----------
+        vertices : SFrame
+            An SFrame containing vertex id columns and optional vertex data columns. Optional.
+        edges : SFrame
+            An SFrame containing source and target id columns and optional edge data columns. Optional.
+        vid_field : str
+            The name of vertex id column in the `vertices` SFrame. Optional
+        src_field : str
+            The name of source id column in the `edges` SFrame. Optional
+        dst_field : str
+            The name of target id column in the `edges` SFrame. Optional
         """
         if (_proxy is None):
             self.__proxy__ = UnityGraphProxy(glconnect.get_client())
+            if vertices is not None:
+                self.__proxy__ = self.add_vertices(vertices, vid_field).__proxy__
+            if edges is not None:
+                self.__proxy__ = self.add_edges(edges, src_field, dst_field).__proxy__
         else:
             self.__proxy__ = _proxy
+        self._vertices = GFrame(self, VERTEX_GFRAME)
+        self._edges = GFrame(self, EDGE_GFRAME)
 
     def __str__(self):
         """Returns a readable string representation summarizing the graph."""
@@ -251,10 +349,79 @@ class SGraph(object):
 
     def __repr__(self):
         """Returns a readable string representation summarizing the graph."""
-        return "SGraph(%s)\nFields:%s" % (str(self.summary()), str(self.get_fields()))
+        return "SGraph(%s)\nVertex Fields:%s\nEdge Fields:%s" % \
+               (str(self.summary()), str(self.get_vertex_fields()), str(self.get_edge_fields()))
+
+    @property
+    def vertices(self):
+        """
+        The special vertex SFrame of the graph for vertex data manipulation.
+
+        Modifying the columns of the returned SFrame will update the vertex data of the graph.
+        To preserve the graph structure, the '__id' column of the returned SFrame is read-only.
+
+        Examples
+        --------
+        >>> import graphlab
+        >>> from graphlab import SGraph, Vertex
+        >>> g = SGraph().add_vertices([Vertex('cat', {'fluffy': 1}),
+                                       Vertex('dog', {'fluffy': 1, 'woof': 1}),
+                                       Vertex('hippo', {})])
+
+        Copy 'woof' into 'bark':
+
+        >>> g.vertices['bark'] = g.vertices['woof']
+
+        Remove 'woof'
+
+        >>> del g.vertices['woof']
+
+        Create a new field 'like_fish'
+
+        >>> g.vertices['like_fish'] = g.vertices['__id'] == 'cat'
+        >>> print g.vertices
+        +-------+--------+------+-----------+
+        |  __id | fluffy | bark | like_fish |
+        +-------+--------+------+-----------+
+        |  dog  |  1.0   | 1.0  |     0     |
+        |  cat  |  1.0   | nan  |     1     |
+        | hippo |  nan   | nan  |     0     |
+        +-------+--------+------+-----------+
+        [3 rows x 4 columns]
+
+        Replace missing values with 0
+
+        >>> for col in g.vertices.column_names():
+        ...     if col != '__id':
+        ...         g.vertices.fillna(col, 0)
+        >>> print g.vertices
+        +-------+--------+------+-----------+
+        |  __id | fluffy | bark | like_fish |
+        +-------+--------+------+-----------+
+        |  dog  |  1.0   | 1.0  |     0     |
+        |  cat  |  1.0   | 0.0  |     1     |
+        | hippo |  0.0   | 0.0  |     0     |
+        +-------+--------+------+-----------+
+        [3 rows x 4 columns]
+        """
+        return self._vertices
+
+    @property
+    def edges(self):
+        """
+        The special edge SFrame of the graph for edge data manipulation.
+
+        Modifying the columns of the returned SFrame will update the edge data of the graph.
+        To preserve the graph structure, the '__src_id', and '__dst_id'
+        columns of the returned SFrame are read-only.
+
+        Please see examples in :py:attr:`~graphlab.SGraph.vertices`.
+        """
+        return self._edges
 
     def summary(self):
-        """Return basic graph statistics in a dictionary.
+        """
+        Return basic graph statistics in a dictionary.
 
         Returns
         -------
@@ -314,7 +481,7 @@ class SGraph(object):
 
         >>> g.get_vertices(fields={'gender': 'M'})
 
-        Return vertices 1, 2, 3 which have the vertex attribute "gender" set to
+        Return vertices 1, 2, and 3 that have the vertex attribute "gender" set to
         "M".
 
         >>> g.get_vertices(ids=[1, 2, 3], fields={'gender': 'M'})
@@ -328,10 +495,11 @@ class SGraph(object):
         if (format == 'sframe'):
             return sf
         elif (format == 'dataframe'):
+            assert HAS_PANDAS, 'Cannot use dataframe because Pandas is not avaiable or version is too low.'
             if sf.num_rows() == 0:
                 return pd.DataFrame()
             else:
-                df = sf.head(sf.num_rows())
+                df = sf.head(sf.num_rows()).to_dataframe()
                 return df.set_index('__id')
         elif (format == 'list'):
             return _dataframe_to_vertex_list(sf.to_dataframe())
@@ -422,15 +590,19 @@ class SGraph(object):
             dst_ids = [None] * len(src_ids)
 
         with cython_context():
-            sf = SFrame(_proxy=self.__proxy__.get_edges(src_ids, dst_ids, fields))
+            if (len(src_ids) + len(dst_ids) + len(fields)) == 0:
+                sf = SFrame(_proxy=self.__proxy__.get_edges_lazy())
+            else:
+                sf = SFrame(_proxy=self.__proxy__.get_edges(src_ids, dst_ids, fields))
 
         if (format == 'sframe'):
             return sf
         if (format == 'dataframe'):
+            assert HAS_PANDAS, 'Cannot use dataframe because Pandas is not avaiable or version is too low.'
             if sf.num_rows() == 0:
                 return pd.DataFrame()
             else:
-                return sf.head(sf.num_rows())
+                return sf.head(sf.num_rows()).to_dataframe()
         elif (format == 'list'):
             return _dataframe_to_edge_list(sf.to_dataframe())
         else:
@@ -467,6 +639,9 @@ class SGraph(object):
             If the arguments do not match the required type.
         """
         sf = _vertex_data_to_sframe(vertices, vid_field)
+        if (sf[_VID_COLUMN]==None).sum() > 0:
+            raise ValueError('Vertex id column cannot contain missing value.'
+                    ' Please use dropna() to drop the missing value from the input and try again.')
 
         with cython_context():
             proxy = self.__proxy__.add_vertices(sf.__proxy__, _VID_COLUMN)
@@ -510,14 +685,35 @@ class SGraph(object):
             If the arguments do not match the required types.
         """
         sf = _edge_data_to_sframe(edges, src_field, dst_field)
+        if (sf[_SRC_VID_COLUMN]==None).sum() > 0:
+            raise ValueError('source vid column cannot contain missing value. '
+                    'Please use dropna() to drop the missing value from the input and try again.')
+        if (sf[_DST_VID_COLUMN]==None).sum() > 0:
+            raise ValueError('target vid column cannot contain missing value. '
+                    'Please use dropna() to drop the missing value from the input and try again.')
+
         with cython_context():
             proxy = self.__proxy__.add_edges(sf.__proxy__, _SRC_VID_COLUMN, _DST_VID_COLUMN)
             return SGraph(_proxy=proxy)
 
     def get_fields(self):
-        """Return a list of vertex and edge attribute fields in the graph."""
+        """
+        Return a list of vertex and edge attribute fields in the graph.
+
+        If a field is common to both vertex and edge attributes, it will
+        show up twice in the returned list.
+        """
+        return self.get_vertex_fields() + self.get_edge_fields()
+
+    def get_vertex_fields(self):
+        """Return a list of vertex attribute fields in the graph."""
         with cython_context():
-            return self.__proxy__.get_vertex_fields() + self.__proxy__.get_edge_fields()
+            return self.__proxy__.get_vertex_fields()
+
+    def get_edge_fields(self):
+        """Return a list of edge attribute fields in the graph."""
+        with cython_context():
+            return self.__proxy__.get_edge_fields()
 
     def select_fields(self, fields):
         """
@@ -549,7 +745,7 @@ class SGraph(object):
             elif f in efields:
                 selected_efields.append(f)
             else:
-                raise ValueError('Field %f not in graph' % f)
+                raise ValueError('Field %s not in graph' % f)
 
         with cython_context():
             proxy = self.__proxy__
@@ -557,84 +753,95 @@ class SGraph(object):
             proxy = proxy.select_edge_fields(selected_efields)
             return SGraph(_proxy=proxy)
 
-    def delete_field(self, field):
-        """
-        Delete a field in the graph. The field is removed from all vertices and
-        edges.
+    def triple_apply(self, triple_apply_fn, mutated_fields, input_fields=None):
+        '''
+        Apply a transform function to each edge and its associated source and target vertices in parallel.
+        Each edge is visited once and in parallel. Modification to vertex data will be protected by lock.
+
+        The effect on the return graph is equivalent to the following pseudo code:
+
+        >>> PARALLEL FOR (source, edge, target) AS triple in G:
+        ...     LOCK (triple.source, triple.target)
+        ...     (source, edge, target) = triple_apply_fn(triple)
+        ...     UNLOCK (triple.source, triple.target)
+        ... END PARALLEL FOR
 
         Parameters
         ----------
-        field : string
-            Name of the field to be deleted.
+        triple_apply_fn : Function : (dict, dict, dict) -> (dict, dict, dict)
+            The function takes as input a tuple of (source_data, edge_data, target_data)
+            and returns a tuple of (new_source_data, new_target_data).
+            All variables in the input and output tuples have dict type.
+
+        mutated_fields : List[str] | str
+            A list of column names for the fields that triple_apply_fn will be mutating.
+            Note: columns that are actually mutated by the triple_apply_fn but not specified in
+            mutated_fields will have undetermined effects.
+
+        input_fields : List[str] | str
+            A list of column names that the triple_apply_fn will have access to. Must be a superset
+            of `mutated_fields`.  Optional, defaults to None, which grant access to all fields.
+
+        Examples
+        --------
+        The following will return a new graph g2 and store the vertex degree
+        count in the "degree" field.
+
+        >>> import graphlab as gl
+        >>> g = gl.load_sgraph('some_graph')
+        >>> vdata = g.get_vertices()
+        >>> vdata['degree'] = 0
+        >>> g = g.add_vertices(vdata, vid_field='__id')
+        >>> def degree_count_fn (src, edge, dst):
+                src['degree'] += 1
+                dst['degree'] += 1
+                return (src, edge, dst)
+        >>> g2 = g.triple_apply(degree_count_fn, mutated_fields=['degree'])
+        >>> print g2.get_vertices(['degree'])
 
         Returns
         -------
         out : SGraph
-            The new graph with field removed.
+            A new SGraph with updated vertex, and edge data.
+            The updated fields are restricted to be the mutated_fields.
 
-        Examples
-        --------
-        >>> g = SGraph().add_vertices([Vertex('cat', {'fluffy': 1}),
-                                     Vertex('dog', {'fluffy': 1, 'woof': 1}),
-                                     Vertex('hippo', {})])
-        >>> g2 = g.delete_field('woof')
-        >>> g2.get_vertices()
-        """
-        vfields = self.__proxy__.get_vertex_fields()
-        efields = self.__proxy__.get_edge_fields()
-        with cython_context():
-            proxy = self.__proxy__
-            if field in vfields:
-                proxy = proxy.delete_vertex_field(field)
-            elif field in efields:
-                proxy = proxy.delete_edge_field(field)
-            else:
-                raise ValueError('Field %f not in graph' % field)
-            return SGraph(_proxy=proxy)
+        Notes
+        -----
+        Currently the triple_apply does not support creating new fields
+        in the lambda function.
+        '''
+        assert inspect.isfunction(triple_apply_fn), "Input must be a function"
+        if not (type(mutated_fields) is list or type(mutated_fields) is str):
+            raise TypeError('mutated_fields must be str or list of str')
+        if not (input_fields is None or type(input_fields) is list or type(input_fields) is str):
+            raise TypeError('input_fields must be str or list of str')
+        if type(mutated_fields) == str:
+            mutated_fields = [mutated_fields]
+        if len(mutated_fields) is 0:
+            raise ValueError('mutated_fields cannot be empty')
+        for f in ['__id', '__src_id', '__dst_id']:
+            if f in mutated_fields:
+                raise ValueError('mutated_fields cannot contain %s' % f)
 
-    def copy_field(self, src_field, new_field):
-        """
-        Create a new field by copying an existing field. All vertices and edges
-        which contain the attribute src_field will now have a new attribute with
-        name new_field and value equal to the src_field attribute.
+        # select input fields
+        g = None
+        if input_fields is None:
+            input_fields = self.get_fields()
+            g = self
+        elif type(input_fields) is str:
+            input_fields = [input_fields]
+            g = self.select_fields(input_fields)
 
-        Parameters
-        ----------
-        src_field : string
-            Name of the source field.
+        # check input fields contains mutated fields
+        if not set(mutated_fields).issubset(input_fields):
+            raise ValueError('Mutated field \"%s\" does not exist in graph' % f)
 
-        new_field : string
-            Name of the destination field.
-
-        Returns
-        -------
-        out : SGraph
-            The new graph with copied field.
-
-        Examples
-        --------
-        >>> g = SGraph().add_vertices([Vertex('cat', {'fluffy': 1}),
-                                     Vertex('dog', {'fluffy': 1, 'woof': 1}),
-                                     Vertex('hippo', {})])
-        >>> g2 = g.copy_field('woof', 'bark')
-        >>> g2.get_vertices()
-        """
-        vfields = self.__proxy__.get_vertex_fields()
-        efields = self.__proxy__.get_edge_fields()
-        with cython_context():
-            proxy = self.__proxy__
-            if src_field in vfields:
-                proxy = proxy.copy_vertex_field(src_field, new_field)
-            elif src_field in efields:
-                proxy = proxy.copy_edge_field(src_field, new_field)
-            else:
-                raise ValueError('Field %f not in graph' % src_field)
-            return SGraph(_proxy=proxy)
+        return SGraph(_proxy=g.__proxy__.lambda_triple_apply(triple_apply_fn, mutated_fields))
 
     def save(self, filename, format='auto'):
         """
         Save the graph to disk. If the graph is saved in binary format, the
-        graph can be loaded using the ``load_graph`` method. Alternatively, it
+        graph can be loaded using the ``load_sgraph`` method. Alternatively, it
         can be saved as "json" to get a human readable / portable graph
         representation.
 
@@ -654,7 +861,7 @@ class SGraph(object):
 
         See Also
         --------
-        load_graph
+        load_sgraph
         """
         if format is 'auto':
             if filename.endswith(('.json', '.json.gz')):
@@ -668,28 +875,38 @@ class SGraph(object):
         with cython_context():
             self.__proxy__.save_graph(make_internal_url(filename), format)
 
-    def show(self, vlabel=None, vcolor=[0.6, 0., 0.6], highlight=[],
-             highlight_color=[1., 0.5, 0.], node_size=300, elabel=None,
-             ecolor='gray', ewidth=3, v_offset=0.03, h_offset=0., arrows=False):
+    @graphlab.canvas.inspect.find_vars
+    def show(self, vlabel=None, vlabel_hover=False, vcolor=[0.522, 0.741, 0.], highlight={},
+             highlight_color=[0.69, 0., 0.498], node_size=300, elabel=None, elabel_hover=False,
+             ecolor=[0.37, 0.33, 0.33], ewidth=1, v_offset=0.03, h_offset=0., arrows=False,
+             vertex_positions=None):
         """
-        Draw the graph using the `NetworkX <http://networkx.github.io/>`_ spring
-        layout. Beacause drawing big graphs is messy and resource consuming,
-        this function only works for graphs with fewer than 1,000 edges.
+        Launch or update GraphLab Canvas and generate specified visualization view of the SGraph.
+        This function only outputs plots for graphs with fewer than 1,000 edges and 1,000 vertices.
 
         Parameters
         ----------
         vlabel : string, optional
-            Field name for the label on each vertex. The default is None, which
-            omits vertex labels. Set to 'id' to use the vertex ID as the label.
+            Field name for the label on each vertex. The default is None,
+            which omits vertex labels. Set to 'id' to use the vertex ID as the
+            label.
 
-        vcolor : list, optional
-            RGB triple for the primary vertex color. Default is orange.
+        vlabel_hover : bool, optional
+            If vlabel_hover is specified, determines how the labels should be presented.
+            The default (False) will cause labels to always be shown.
+            Specifying True will show labels only on mouse hover.
 
-        highlight : list, optional
-            Vertex IDs to highlight in a different color.
+        vcolor : list of float, optional
+            RGB triple for the primary vertex color. Default is green ([0.522, 0.741, 0.]).
 
-        highlight_color : list, optional
-            RGB triple for the color of highlighted vertices. Default is purple.
+        highlight : dict or list or SArray, optional
+            As a dict, mapping of Vertex ID to RGB color triple (list of float, as in vcolor).
+            As a list or SArray (DEPRECATED): Vertex IDs to highlight in a different color.
+
+        highlight_color : list of float, optional
+            RGB triple for the color of highlighted vertices, when the highlight parameter
+            is a list or SArray. Default is fuchsia ([0.69, 0., 0.498]). For fine-grained control over vertex coloring,
+            use the highlight parameter with a dictionary of Vertex IDs and color values.
 
         node_size : int, optional
             Size of plotted vertices.
@@ -697,8 +914,13 @@ class SGraph(object):
         elabel : string, optional
             Field name for the label on each edge.
 
+        elabel_hover : bool, optional
+            If elabel_hover is specified, determines how the labels should be presented.
+            The default (False) will cause labels to always be shown.
+            Specifying True will show labels only on mouse hover.
+
         ecolor : string, optional
-            Edge color.
+            RGB triple for edge color. Default is grey ([0.37, 0.33, 0.33]).
 
         ewidth : int, optional
             Edge width.
@@ -714,76 +936,52 @@ class SGraph(object):
             width to the right. Default is 0.0.
 
         arrows : bool, optional
-            If True, draw 'arrows' indicating edge direction. In place of real
-            arrows, NetworkX indicates edge direction by drawing a wider line
-            segment adjacent to the source node.
+            If True, draw arrows indicating edge direction.
+
+        vertex_positions : tuple, optional
+            If a 2-element tuple of column names in self.vertices is specified,
+            those two columns will be used as the X and Y coordinates of vertices
+            in the graph layout. If not specified, vertices will be arranged
+            according to a standard graph layout algorithm without regard to
+            vertex or edge attributes.
+
+        Examples
+        --------
+        To plot a graph with fewer than 1,000 vertices and edges, call .show and
+        provide relevant parameters.
+
+        >>> sg = graphlab.SGraph()
+        >>> sg = sg.add_edges([graphlab.Edge(i, i+1) for i in range(5)])
+        >>> sg.show(highlight=[2,3], elabel='__src_id', elabel_hover=False)
+
+        Notes
+        -----
+        Graphs larger than 1,000 vertices and edges cannot be displayed as-is.
+        For such graphs, construct a subgraph by selecting some vertices and
+        edges, then call .show on the result.
         """
-
-        ## Error trapping
-        if not _HAS_MATPLOTLIB:
-            raise RuntimeError("Was unable to import matplotlib or networkx. Unable to show graph.")
-
-        maxedges = 1000
-        num_edges = self.summary()["num_edges"]
-        if (num_edges > maxedges):
-            raise RuntimeError("Cannot show graph with too many edges")
-
-        ## Make the NetworkX graph
-        nxgraph = nx.DiGraph()
-        vertices = self.get_vertices(format='list')
-        edges = self.get_edges(format='list')
-        node_color = []
-        nodelist = []
-
-        for v in vertices:
-            nodelist.append(v.vid)
-            nxgraph.add_node(v.vid)
-
-            if v.vid in highlight:
-                node_color.append(vcolor)
-            else:
-                node_color.append(highlight_color)
-
-        for e in edges:
-            nxgraph.add_edge(e.src_vid, e.dst_vid)
-
-        pos = nx.spring_layout(nxgraph)
-
-        ## Draw the graph
-        nx.draw_networkx_nodes(nxgraph, pos, nodelist=nodelist,
-                               node_color=node_color, node_size=node_size,
-                               alpha=0.7)
-
-        nx.draw_networkx_edges(nxgraph, pos, edge_color=ecolor, width=ewidth,
-                               alpha=0.5, arrows=arrows)
-
-        ## Draw vertex labels
-        if vlabel:
-            # get offset positions
-            pos_label = {}
-            for k, v in pos.items():
-                xcoord = max(0., pos[k][0] + h_offset)
-                ycoord = max(0., pos[k][1] + v_offset)
-                pos_label[k] = [xcoord, ycoord]
-
-            # plot labels
-            if vlabel == "id":
-                names = {v.vid: v.vid for v in vertices}
-                nx.draw_networkx_labels(nxgraph, pos_label, labels=names,
-                                        font_color='b')
-            else:
-                names = {v.vid: v.attr[vlabel] for v in vertices}
-                nx.draw_networkx_labels(nxgraph, pos_label, labels=names,
-                                        font_color='b')
-
-        # Draw edge labels
-        if elabel:
-            names = dict([((e.src_vid, e.dst_vid), e.attr[elabel]) for e in edges])
-            nx.draw_networkx_edge_labels(nxgraph, pos, edge_labels=names,
-                                         font_color='b')
-
-        plt.axis('off')
-        plt.show()
+        if isinstance(highlight, SArray):
+            # convert to list
+            highlight = list(highlight)
+        if isinstance(highlight, list):
+            # convert to dict
+            highlight_color = [highlight_color] * len(highlight)
+            highlight = dict(zip(highlight, highlight_color))
+        return graphlab.canvas._show(variable=self, params={
+            'vlabel': vlabel,
+            'vlabel_hover': vlabel_hover,
+            'vcolor': vcolor,
+            'highlight': highlight,
+            'node_size': node_size,
+            'elabel': elabel,
+            'elabel_hover': elabel_hover,
+            'ecolor': ecolor,
+            'ewidth': ewidth,
+            'v_offset': v_offset,
+            'h_offset': h_offset,
+            'arrows': arrows,
+            'vertex_positions': vertex_positions
+        })
 
     def get_neighborhood(self, ids, radius=1, full_subgraph=True):
         """
@@ -863,6 +1061,12 @@ class SGraph(object):
 #/*                                                                        */
 #/**************************************************************************/
 def load_graph(filename, format='binary'):
+    import warnings
+    warnings.warn("load_graph has been renamed to load_sgraph. This function will be removed in the next release.", PendingDeprecationWarning)
+    return load_sgraph(filename, format=format)
+
+
+def load_sgraph(filename, format='binary'):
     """
     Load graph from text file or previously saved graph binary.
 
@@ -910,6 +1114,7 @@ def _vertex_list_to_dataframe(ls, id_column_name):
     """
     Convert a list of vertices into dataframe.
     """
+    assert HAS_PANDAS, 'Cannot use dataframe because Pandas is not avaiable or version is too low.'
     cols = reduce(set.union, (set(v.attr.keys()) for v in ls))
     df = pd.DataFrame({id_column_name: [v.vid for v in ls]})
     for c in cols:
@@ -921,6 +1126,7 @@ def _edge_list_to_dataframe(ls, src_column_name, dst_column_name):
     """
     Convert a list of edges into dataframe.
     """
+    assert HAS_PANDAS, 'Cannot use dataframe because Pandas is not avaiable or version is too low.'
     cols = reduce(set.union, (set(e.attr.keys()) for e in ls))
     df = pd.DataFrame({
         src_column_name: [e.src_vid for e in ls],
@@ -965,7 +1171,7 @@ def _vertex_data_to_sframe(data, vid_field):
     the id column.
     The returned sframe will have id column name '__id'
     """
-    if (type(data) == SFrame):
+    if isinstance(data, SFrame):
         # '__id' already in the sframe, and it is ok to not specify vid_field
         if vid_field is None and _VID_COLUMN in data.column_names():
             return data
@@ -978,7 +1184,7 @@ def _vertex_data_to_sframe(data, vid_field):
         return SFrame(_vertex_list_to_dataframe(data, '__id'))
     elif (type(data) == list):
         return SFrame(_vertex_list_to_dataframe(data, '__id'))
-    elif (type(data) == pd.DataFrame):
+    elif (HAS_PANDAS and type(data) == pd.DataFrame):
         if vid_field is None:
             # using the dataframe index as vertex id
             if data.index.is_unique:
@@ -997,6 +1203,8 @@ def _vertex_data_to_sframe(data, vid_field):
                         try specifying vid_field name to use a column for vertex ids.")
         else:
             sf = SFrame(data)
+            if _VID_COLUMN in sf.column_names():
+                raise ValueError('%s reserved vid column name already exists in the SFrame' % _VID_COLUMN)
             sf.rename({vid_field: _VID_COLUMN})
             return sf
     else:
@@ -1009,7 +1217,7 @@ def _edge_data_to_sframe(data, src_field, dst_field):
     the source and target id column.
     The returned sframe will have id column name '__src_id', '__dst_id'
     """
-    if type(data) == SFrame:
+    if isinstance(data, SFrame):
         # '__src_vid' and '__dst_vid' already in the sframe, and
         # it is ok to not specify src_field and dst_field
         if src_field is None and dst_field is None and \
@@ -1021,15 +1229,31 @@ def _edge_data_to_sframe(data, src_field, dst_field):
         if dst_field is None:
             raise ValueError("dst_field must be specified for SFrame input")
         data_copy = copy.copy(data)
-        data_copy.rename({src_field: _SRC_VID_COLUMN, dst_field: _DST_VID_COLUMN})
+        if src_field == _DST_VID_COLUMN and dst_field == _SRC_VID_COLUMN:
+            # special case when src_field = "__dst_id" and dst_field = "__src_id"
+            # directly renaming will cause name collision
+            dst_id_column = data_copy[_DST_VID_COLUMN]
+            del data_copy[_DST_VID_COLUMN]
+            data_copy.rename({_SRC_VID_COLUMN: _DST_VID_COLUMN})
+            data_copy[_SRC_VID_COLUMN] = dst_id_column
+        else:
+            data_copy.rename({src_field: _SRC_VID_COLUMN, dst_field: _DST_VID_COLUMN})
         return data_copy
-    elif type(data) == pd.DataFrame:
+    elif HAS_PANDAS and type(data) == pd.DataFrame:
         if src_field is None:
             raise ValueError("src_field must be specified for Pandas input")
         if dst_field is None:
             raise ValueError("dst_field must be specified for Pandas input")
         sf = SFrame(data)
-        sf.rename({src_field: _SRC_VID_COLUMN, dst_field: _DST_VID_COLUMN})
+        if src_field == _DST_VID_COLUMN and dst_field == _SRC_VID_COLUMN:
+            # special case when src_field = "__dst_id" and dst_field = "__src_id"
+            # directly renaming will cause name collision
+            dst_id_column = data_copy[_DST_VID_COLUMN]
+            del sf[_DST_VID_COLUMN]
+            sf.rename({_SRC_VID_COLUMN: _DST_VID_COLUMN})
+            sf[_SRC_VID_COLUMN] = dst_id_column
+        else:
+            sf.rename({src_field: _SRC_VID_COLUMN, dst_field: _DST_VID_COLUMN})
         return sf
     elif type(data) == Edge:
         return SFrame(_edge_list_to_dataframe([data], _SRC_VID_COLUMN, _DST_VID_COLUMN))
@@ -1037,3 +1261,7 @@ def _edge_data_to_sframe(data, src_field, dst_field):
         return SFrame(_edge_list_to_dataframe(data, _SRC_VID_COLUMN, _DST_VID_COLUMN))
     else:
         raise TypeError('Edges type %s is Not supported.' % str(type(data)))
+
+## Hack: overriding GFrame class name to make it appears as SFrame##
+GFrame.__name__ = SFrame.__name__
+GFrame.__module__ = SFrame.__module__
