@@ -15,9 +15,9 @@ ID. The fastest way to use the GraphLab recommender system via the
 .. sourcecode:: python
 
     >>> from graphlab import recommender, SFrame
-    >>> data = SFrame('s3:///graphlab-sample-data/example.csv')
-    >>> m = recommender.create(data, user_column='user_id', item_column='item_id')
-    >>> recs = m.recommend(users=['u1', 'u3', 'u15'])
+    >>> data = SFrame('s3://GraphLab-Datasets/audioscrobbler')
+    >>> m = recommender.create(data)
+    >>> recs = m.recommend()
     >>> print recs
 
 When no ``target_column`` is provided, then by default, this returns a
@@ -48,7 +48,7 @@ as ``n_factors`` or ``regularization`` when using a
                                method='matrix_factorization',
                                n_factors=7, regularization=0.05, nmf=True)
 
-All model objects in the :mod:`~graphlab.recommender` module expose a common set of
+All model objects in the :py:mod:`~graphlab.recommender` module expose a common set of
 methods, such as :py:func:`~graphlab.recommender.MatrixFactorizationModel.recommend`
 and :py:func:`~graphlab.recommender.MatrixFactorizationModel.evaluate`.
 
