@@ -5,10 +5,9 @@ import graphlab as _graphlab
 from graphlab.toolkits.model import Model
 from graphlab.data_structures.sarray import SArray as _SArray
 from graphlab.data_structures.sframe import SFrame as _SFrame
-import pandas as _pd
+from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
+from graphlab.deps import numpy as _numpy, HAS_NUMPY as _HAS_NUMPY
 import json as _json
-import array
-import numpy as _np
 import logging as _logging
 
 class RecommenderModel(Model):
@@ -25,11 +24,13 @@ class RecommenderModel(Model):
 
         Returns
         -------
-        out : dict
-            A dictionary that maps model options to the values used during
-            training.
+        out : list
+            A list of fields that can be queried using the ``get`` method.
         """
-        return self._get_current_options().keys()
+
+        opts = {'model': self.__proxy__}
+        response = _graphlab.toolkits.main.run('recsys_list_fields', opts)
+        return [s for s in response['value'] if not s.startswith("_")]
 
     def get(self, field):
         """
@@ -42,12 +43,47 @@ class RecommenderModel(Model):
 
         Returns
         -------
-        out : string
-            The current value of the requested field.
+        The current value of the requested field.
+
+        Example
+        -------
+
+        m = gl.recommender.create(data, "user_id", "movie_id", "rating",
+                                  method = "matrix_factorization")
+
+        d = m.get("coefficients")
+
+        U1 = d['user_id']
+        U2 = d['movie_id']
+
         """
+
         opts = {'model': self.__proxy__, 'field': field}
-        response = _graphlab.toolkits.main.run('recsys_get_option_value', opts)
-        return response['value']
+        response = _graphlab.toolkits.main.run('recsys_get_value', opts)
+
+        # Unpack the response
+
+        result_type = response["__result_type__"]
+
+        if result_type == "value":
+            return response["value"]
+
+        elif result_type == "dict":
+
+            # Unpack the dictionary
+            keys = response["__keys__"]
+
+            def _translate_type(x):
+                if isinstance(x, _graphlab.cython.cy_sframe.UnitySFrameProxy):
+                    return _graphlab.SFrame(_proxy = x)
+                else:
+                    return x
+
+            return dict( (k, _translate_type(response[k])) for k in keys)
+
+        else:
+            assert False, "Result type not specified."
+
 
     def summary(self):
         """Returns a summary of statistics about the model, the data
@@ -225,7 +261,8 @@ class RecommenderModel(Model):
         response = _graphlab.toolkits.main.run('recsys_set_current_options', opts)
         return response
 
-    def score(self, dataset):
+    def score(self, dataset,
+              new_observation_data=None, new_user_data=None, new_item_data=None):
         """
         Return a score prediction for the user ids and item ids in the provided
         data set.
@@ -235,19 +272,71 @@ class RecommenderModel(Model):
         dataset : SFrame
             Dataset in the same form used for training.
 
+        new_observation_data: SFrame, optional
+            ``new_observation_data`` gives additional observation data
+            to the model, which may be used by the models to improve
+            score accuracy.  Must be in the same format as the
+            observation data passed to ``create``.  How this data is
+            used varies by model.
+
+        new_user_data: SFrame, optional
+            ``new_user_data`` may give additional user data to the
+            model.  If present, scoring is done with reference to this
+            new information.  If there is any overlap with the side
+            information present at training time, then this new side
+            data is preferred.  Must be in the same format as the user
+            data passed to ``create``.
+
+        new_item_data: SFrame, optional
+            ``new_item_data`` may give additional item data to the
+            model.  If present, scoring is done with reference to this
+            new information.  If there is any overlap with the side
+            information present at training time, then this new side
+            data is preferred.  Must be in the same format as the item
+            data passed to ``create``.
+
         Returns
         -------
         out : SArray
-            An SArray with scores for each given user-item pair predicted by
-            the model.
+            An SArray with predicted scores for each given observation
+            predicted by the model.
         """
         _mt._get_metric_tracker().track('toolkit.recsys.score')
 
-        if type(dataset) == _pd.DataFrame:
+        if _HAS_PANDAS and type(dataset) == _pandas.DataFrame:
             dataset = _SFrame(dataset)
 
+        if new_observation_data is None:
+            new_observation_data = _SFrame()
+        if new_user_data is None:
+            new_user_data = _SFrame()
+        if new_item_data is None:
+            new_item_data = _SFrame()
+
+        if (_HAS_PANDAS and isinstance(new_observation_data, _pandas.DataFrame)) or (_HAS_NUMPY and isinstance(new_observation_data, _numpy.ndarray)):
+            new_observation_data = _SFrame(new_observation_data)
+        if (_HAS_PANDAS and isinstance(new_user_data, _pandas.DataFrame)) or (_HAS_NUMPY and isinstance(new_user_data, _numpy.ndarray)):
+            new_user_data = _SFrame(new_user_data)
+        if (_HAS_PANDAS and isinstance(new_item_data, _pandas.DataFrame)) or (_HAS_NUMPY and isinstance(new_item_data, _numpy.ndarray)):
+            new_item_data = _SFrame(new_item_data)
+
+        def check_type(arg, arg_name, required_type, allowed_types):
+            if not isinstance(arg, required_type):
+                raise TypeError("Parameter " + arg_name + " must be of type(s) "
+                                + (", ".join(allowed_types))
+                                + "; Type '" + str(type(arg)) + "' not recognized.")
+
+        check_type(new_observation_data, "new_observation_data", _SFrame, ["SFrame"])
+        check_type(new_user_data, "new_user_data", _SFrame, ["SFrame"])
+        check_type(new_item_data, "new_item_data", _SFrame, ["SFrame"])
+
         # Get metadata from C++ object
-        opts = {'data_to_predict': dataset, 'model': self.__proxy__}
+        opts = {'data_to_predict': dataset,
+                'model': self.__proxy__,
+               'new_data': new_observation_data,
+               'new_user_data': new_user_data,
+               'new_item_data': new_item_data
+                }
 
         # Call the C++ function for recommender_model
         response = _graphlab.toolkits.main.run('recsys_predict', opts)
@@ -256,40 +345,80 @@ class RecommenderModel(Model):
 
     def recommend(self, users=None, k=10, exclude=None, items=None,
                   new_observation_data=None, new_user_data=None, new_item_data=None,
-                  exclude_known=True):
+                  exclude_known=True,
+                  verbose=True):
         """
         Recommends the k-highest scored items for each user in users.
 
         Parameters
         ----------
         users:  SArray or list; optional
-            An :class:`~graphlab.SArray` (or list) of users for which to make recommendations.  If 'None', then recommend(...) generates predictions for all users in the training set.
+            An :class:`~graphlab.SArray` (or list) of users for which
+            to make recommendations.  If 'None', then recommend(...)
+            generates predictions for all users in the training set.
 
         k: int, optional
             The number of recommendations to generate for each user.
 
         items: SArray, optional
-            To restrict the set of items under consideration, the user may pass in an SArray of items from which the recommended items will be chosen.  This allows one to choose only items that are, for instance, in a particular genre or category.   The default is that all items are under consideration.
+            To restrict the set of items under consideration, the user
+            may pass in an SArray of items from which the recommended
+            items will be chosen.  This allows one to choose only
+            items that are, for instance, in a particular genre or
+            category.  The default is that all items are under
+            consideration.
 
         new_observation_data: SFrame, optional
-            ``new_observation_data`` may give additional observation data to the model.  Any overlap with the training data will be ignored.  Must be in the same format as the observation data passed to ``create``.
+            ``new_observation_data`` gives additional observation data
+            to the model, which may be used by the models to improve
+            score and recommendation accuracy.  Must be in the same
+            format as the observation data passed to ``create``.  How
+            this data is used varies by model.
 
         new_user_data: SFrame, optional
-            ``new_user_data`` may give additional user data to the model.  Any overlap with the training data will be ignored.  Must be in the same format as the user data passed to ``create``.
+            ``new_user_data`` may give additional user data to the
+            model.  If present, scoring is done with reference to this
+            new information.  If there is any overlap with the side
+            information present at training time, then this new side
+            data is preferred.  Must be in the same format as the user
+            data passed to ``create``.
 
         new_item_data: SFrame, optional
-            ``new_item_data`` may give additional item data to the model.  Any overlap with the training data will be ignored.  Must be in the same format as the item data passed to ``create``.
+            ``new_item_data`` may give additional item data to the
+            model.  If present, scoring is done with reference to this
+            new information.  If there is any overlap with the side
+            information present at training time, then this new side
+            data is preferred.  Must be in the same format as the item
+            data passed to ``create``.
 
         exclude: SFrame, optional
-            An :class:`~graphlab.SFrame` of user / item pairs.  The column names must be equal to the user and item columns of the main data, and it provides the model with user/item pairs to exclude from the recommendations.  These user-item-pairs are always excluded from the predictions, even if exclude_known is False.
+            An :class:`~graphlab.SFrame` of user / item pairs.  The
+            column names must be equal to the user and item columns of
+            the main data, and it provides the model with user/item
+            pairs to exclude from the recommendations.  These
+            user-item-pairs are always excluded from the predictions,
+            even if exclude_known is False.
 
         exclude_known: bool
-            By default, all user-item interactions previously seen in the training data, or in any new data provided using new_observation_data=..., are excluded from the recommendations.  Passing in ``exclude_known = False`` overrides this behavior.
+            By default, all user-item interactions previously seen in
+            the training data, or in any new data provided using
+            new_observation_data=..., are excluded from the
+            recommendations.  Passing in ``exclude_known = False``
+            overrides this behavior.
+
+        verbose: bool
+            If True, print the progress of generating recommendation.
 
         Returns
         -------
         out : SFrame
-            A SFrame with the top ranked items for each user. The columns are: ``user_column``, ``item_column``, *score*, and *rank*, where ``user_column`` and ``item_column`` match the user and item column names specified at training time.  The rank column is between 1 and ``k`` and gives the relative score of that item.  The value of score depends on the method used for recommendations.
+            A SFrame with the top ranked items for each user. The
+            columns are: ``user_column``, ``item_column``, *score*,
+            and *rank*, where ``user_column`` and ``item_column``
+            match the user and item column names specified at training
+            time.  The rank column is between 1 and ``k`` and gives
+            the relative score of that item.  The value of score
+            depends on the method used for recommendations.
 
 
         """
@@ -309,23 +438,23 @@ class RecommenderModel(Model):
         if new_item_data is None:
             new_item_data = _SFrame()
 
-        if isinstance(users, list) or isinstance(users, _np.ndarray):
+        if isinstance(users, list) or (_HAS_NUMPY and isinstance(users, _numpy.ndarray)):
             users = _SArray(users)
-        if isinstance(exclude, _pd.DataFrame) or isinstance(exclude, _np.ndarray):
+        if (_HAS_PANDAS and isinstance(exclude, _pandas.DataFrame)) or (_HAS_NUMPY and isinstance(exclude, _numpy.ndarray)):
             exclude = _SFrame(exclude)
-        if isinstance(items, list) or isinstance(items, _np.ndarray):
+        if isinstance(items, list) or (_HAS_NUMPY and isinstance(items, _numpy.ndarray)):
             items = _SArray(items)
-        if isinstance(new_observation_data, _pd.DataFrame) or isinstance(new_observation_data, _np.ndarray):
+        if (_HAS_PANDAS and isinstance(new_observation_data, _pandas.DataFrame)) or (_HAS_NUMPY and isinstance(new_observation_data, _numpy.ndarray)):
             new_observation_data = _SFrame(new_observation_data)
-        if isinstance(new_user_data, _pd.DataFrame) or isinstance(new_user_data, _np.ndarray):
+        if (_HAS_PANDAS and isinstance(new_user_data, _pandas.DataFrame)) or (_HAS_NUMPY and isinstance(new_user_data, _numpy.ndarray)):
             new_user_data = _SFrame(new_user_data)
-        if isinstance(new_item_data, _pd.DataFrame) or isinstance(new_item_data, _np.ndarray):
+        if (_HAS_PANDAS and isinstance(new_item_data, _pandas.DataFrame)) or (_HAS_NUMPY and isinstance(new_item_data, _numpy.ndarray)):
             new_item_data = _SFrame(new_item_data)
 
         def check_type(arg, arg_name, required_type, allowed_types):
             if not isinstance(arg, required_type):
                 raise TypeError("Parameter " + arg_name + " must be of type(s) "
-                                + (", ".join(allowed_types) )
+                                + (", ".join(allowed_types))
                                 + "; Type '" + str(type(arg)) + "' not recognized.")
 
         check_type(users, "users", _SArray, ["SArray", "list", "numpy.ndarray"])
@@ -343,8 +472,8 @@ class RecommenderModel(Model):
                'new_data': new_observation_data,
                'new_user_data': new_user_data,
                'new_item_data': new_item_data,
-               'exclude_known': exclude_known*1}
-        response = _graphlab.toolkits.main.run('recsys_recommend', opt)
+               'exclude_known': exclude_known}
+        response = _graphlab.toolkits.main.run('recsys_recommend', opt, verbose=verbose)
         recs = _SFrame(None, _proxy=response['data'])
         return recs
 
@@ -369,7 +498,7 @@ class RecommenderModel(Model):
         response = _graphlab.toolkits.main.run("recsys_get_stats", opts)
         return response
 
-    def _evaluate_precision_recall(self, dataset, cutoffs=[5, 10, 15], skip_set=None):
+    def _evaluate_precision_recall(self, dataset, cutoffs=[5, 10, 15], skip_set=None, exclude_known=True, verbose=True):
         """
         Compute a model's precision and recall for a particular dataset.
 
@@ -384,6 +513,13 @@ class RecommenderModel(Model):
             A list of cutoff values for which one wants to evaluate precision
             and recall, i.e. the value of k in precision@k.
 
+        skip_set: SFrame
+            Pass to recommend() as exclude
+
+        exclude_known : bool
+            Pass to recommend() as exclude_known
+            If True, exclude training item from recommendation.
+
         Returns
         -------
         out : SFrame
@@ -394,12 +530,12 @@ class RecommenderModel(Model):
         users = dataset[self.get('user_column')].unique()
 
         dataset = dataset[[self.get('user_column'), self.get('item_column')]]
-        
-        recs = self.recommend(users=users, k=max(cutoffs), exclude=skip_set)
+
+        recs = self.recommend(users=users, k=max(cutoffs), exclude=skip_set, exclude_known=exclude_known, verbose=verbose)
         pr = _graphlab.recommender.util.precision_recall_by_user(dataset, recs, cutoffs)
         return pr
 
-    def _evaluate_rmse(self, dataset):
+    def _evaluate_rmse(self, dataset, target_column):
         """
         Evaluate the prediction error for each user-item pair in the given data
         set.
@@ -409,6 +545,9 @@ class RecommenderModel(Model):
         dataset : SFrame/pandas.DataFrame
             A DataFrame in the same format as the one used during training.
 
+        target_column : str
+            The target column to evaluate rmse on.
+
         Returns
         -------
         out : dict
@@ -416,12 +555,9 @@ class RecommenderModel(Model):
             are SFrames; and *rmse_overall*, which is a float.
         """
 
-        if type(dataset) == _pd.DataFrame:
+        if _HAS_PANDAS and type(dataset) == _pandas.DataFrame:
             dataset = _SFrame(dataset)
 
-        assert 'target_column' in self.list_fields(), \
-               'RMSE evaluation only valid for models trained using a target.'
-        target_column = self.get('target_column')
         assert target_column in dataset.column_names(), \
                'Provided dataset must contain a target column with the same \
                 name as the target used during training.'
@@ -450,7 +586,10 @@ class RecommenderModel(Model):
                 'rmse_by_item': rmse_by_item,
                 'rmse_overall': overall_rmse}
 
-    def evaluate(self, dataset, verbose=False, **kwargs):
+    def evaluate(self, dataset, metric='auto',
+                 exclude_known_for_precision_recall=True,
+                 target_column=None,
+                 verbose=False, **kwargs):
         r"""Evaluate the model's ability to make predictions for another
         dataset. If the model is trained to predict a particular target, the
         default metric used for model comparison is root-mean-squared error
@@ -470,6 +609,25 @@ class RecommenderModel(Model):
             An SFrame or DataFrame that is in the same format as provided for
             training.
 
+        metric : str, {'auto', 'rmse', 'precision_recall'}
+            Metric for the evaluation. The default automatically choose
+            'rmse' for models trained with a target_column, and 'precision_recall'
+            otherwise.
+
+        exclude_known_for_precision_recall : bool, optional
+            A useful option for evaluating precision-recall.
+            Recommender models have the option to exclude items seen in the training
+            data from the final recommendation list.
+            Set this option to True when evaluating on test data,
+            and False when evaluating precision-recall on training data.
+
+        target_column : str, optional
+            The target column for evaluating rmse.
+            If the model is trained with a target column, the default
+            is to using the same column.
+            If the model is trained without a target column, this option
+            must provided by user.
+
         verbose : bool, optional
             Enables verbose output. Default is verbose.
 
@@ -496,16 +654,29 @@ class RecommenderModel(Model):
 
         _mt._get_metric_tracker().track('toolkit.recsys.evaluate')
 
+        if metric is 'auto':
+            if self.get('target_column') is None or self.get('target_column') == '':
+                metric = 'precision_recall'
+            else:
+                metric = 'rmse'
+
         # If the model does not have a target column, compute prec-recall.
-        if self.get('target_column') is None or self.get('target_column') == '':
-            results = self._evaluate_precision_recall(dataset, **kwargs)
+        if metric == 'precision_recall':
+            results = self._evaluate_precision_recall(dataset,
+                                                      exclude_known=exclude_known_for_precision_recall,
+                                                      verbose=verbose,
+                                                      **kwargs)
             if verbose:
                 print "\nPrecision and recall summary statistics by cutoff"
-                print results.groupby('cutoff',
-                        {'mean_precision':_graphlab.aggregate.AVG('precision'),
-                         'mean_recall':_graphlab.aggregate.AVG('recall')}).topk('cutoff', reverse=True)
-        else:
-            results = self._evaluate_rmse(dataset)
+                print results.groupby('cutoff', {'mean_precision': _graphlab.aggregate.AVG('precision'),
+                                                 'mean_recall': _graphlab.aggregate.AVG('recall')}).topk('cutoff', reverse=True)
+        elif metric == 'rmse':
+            if target_column is None:
+                target_column = self.get('target_column')
+            if target_column is None or target_column == "":
+                raise ValueError('Target column cannot be None. Please provide the target_column if the model \
+                    is not trained with target_column')
+            results = self._evaluate_rmse(dataset, target_column)
             if verbose:
                 print "\nOverall RMSE: ", results['rmse_overall']
                 print "\nPer User RMSE (best)"
@@ -516,6 +687,7 @@ class RecommenderModel(Model):
                 print results['rmse_by_item'].topk('rmse', 1, reverse=True)
                 print "\nPer Item RMSE (worst)"
                 print results['rmse_by_item'].topk('rmse', 1)
+        else:
+            raise ValueError('Unknown evaluation metric %s, supported metrics are [\"rmse\", \"precision_recall\"]' % metric)
 
         return results
-
