@@ -3,9 +3,10 @@ Methods for creating and using a linear regression model.
 """
 import graphlab.connect as _mt
 import graphlab as _graphlab
-from graphlab.toolkits.regression.regression import RegressionModel
+from graphlab.toolkits.supervised_learning import SupervisedLearningModel
+from graphlab.data_structures.sframe import SArray as _SArray
 from graphlab.data_structures.sframe import SFrame as _SFrame
-from pandas import DataFrame as _DataFrame
+from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
 from graphlab import vowpal_wabbit as _vw
 
 DEFAULT_SOLVER_OPTIONS = {
@@ -15,9 +16,14 @@ DEFAULT_SOLVER_OPTIONS = {
 'mini_batch_size': 1,
 'max_iterations': 10}
 
+DEFAULT_HYPER_PARAMETER_RANGE = {
+    'L1_penalty' : [0.0, 0.001, 0.01, 0.1, 1.0, 10.0, 100.0],
+    'L2_penalty' : [0.0, 0.001, 0.01, 0.1, 1.0, 10.0, 100.0]
+}
 
 def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
-           solver='auto', solver_options=None, verbose=True):
+    solver='auto', feature_rescaling=True, solver_options=None, verbose=True):
+
     """
     Create a :class:`~graphlab.linear_regression.LinearRegressionModel` to
     predict a scalar target variable as a linear function of one or more
@@ -31,7 +37,7 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
 
     Parameters
     ----------
-    dataset : pandas.DataFrame/SFrame
+    dataset : SFrame 
         The dataset to use for training the model.
 
     target : string
@@ -40,18 +46,26 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
     features : list[string], optional
         Names of the columns containing features. 'None' (the default) indicates
         that all columns except the target variable should be used as features.
-        Each feature column can be of one of the following types:
+        
+        The features are columns in the input SFrame that can be of the
+        following types:
 
         - *Numeric*: values of numeric type integer or float.
 
         - *Categorical*: values of type string.
 
-        - *List*: list of numeric (integer or float) values. Each list element
+        - *Array*: list of numeric (integer or float) values. Each list element
           is treated as a separate feature in the model.
 
-        - *Dictionary*: key-value pairs with numeric (integer or float) values.
-          Each key of a dictionary is treated as a categorical variable in the
-          model.
+        - *Dictionary*: key-value pairs with numeric (integer or float) values
+          Each key of a dictionary is treated as a separate feature and the
+          value in the dictionary corresponds to the value of the feature. 
+          Dictionaries are ideal for representing sparse data.
+
+        Columns of type *list* are not supported. Convert such feature
+        columns to type array if all entries in the list are of numeric
+        types. If the lists contain data of mixed types, separate
+        them out into different columns.
 
     L2_penalty : float, optional
         Weight on the L2-regularizer of the model. The larger this weight, the
@@ -82,6 +96,12 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
         - *fista*: accelerated gradient descent
         - *sgd*: stochastic gradient descent
 
+    feature_rescaling: boolean, optional
+        Feature rescaling is the process of scaling individual features to
+        have unit L2-norm. This process is particularly useful when features
+        vary widely in their ranges.  (Not applicable when solver = 'vw')
+
+
     solver_options : dict, optional
         Solver options. The options and their default values are as follows.
 
@@ -98,7 +118,7 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
         +-----------------------+---------+------------------------------------------+
         | mini_batch_size       |     1   | Number of mini-batch examples (sgd only) |
         +-----------------------+---------+------------------------------------------+
-        | step_size             |     1.0 | Initial solver step size                 |
+        | step_size             |     1.0 | Initial solver step size (fista, sgd)    |
         +-----------------------+---------+------------------------------------------+
 
     verbose : bool, optional (default True)
@@ -117,20 +137,10 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
       variables, while the first category encountered in the data is used as the
       baseline.
 
-    - For prediction and evaluation of linear regression models with
-      categorical variables, test datasets cannot contain categories that were
-      not present during train time.
-
     - For prediction and evaluation of linear regression models with sparse
       dictionary inputs, new keys/columns that were not seen during training
       are silently ignored.
 
-    - L2 and L1 regularization typically work best if the features are
-      standardized before model training by subtracting the mean of column and
-      dividing by the standard deviation. The regression model does *not* do
-      this automatically; use SFrame and SArray methods to transform the data
-      prior to training.
-
     - If the 'vw' solver is used, the model returns only predictions; it does
       not allow inspection of estimated effect sizes.
 
@@ -139,6 +149,9 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
     - A constant term is automatically added for the model intercept. This term
       is not regularized.
 
+    - For solver='vw', the "step_size" option is the "learning_rate" parameter
+      in the  :class:`~graphlab.vowpal_wabbit.VWModel`.
+
     Examples
     --------
     *Training*
@@ -246,9 +259,10 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
       <https://github.com/JohnLangford/vowpal_wabbit/wiki>`_
     """
 
+    print "Preparing the data..."
     _mt._get_metric_tracker().track('toolkit.regression.linear_regression.create')
 
-    if not isinstance(dataset, (_DataFrame, _SFrame)):
+    if not (isinstance(dataset, _SFrame) or (_HAS_PANDAS and isinstance(dataset, _pandas.DataFrame))):
         raise TypeError('Input dataset must be an SFrame or a pandas dataframe.')
 
     if type(dataset) != _SFrame:
@@ -270,8 +284,8 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
 
     # If features is None, then select all other columns in the table.
     if features is None:
-      features = dataset.column_names()
-      features.remove(target)
+        features = dataset.column_names()
+        features.remove(target)
 
     # Make sure that features are of type strings.
     if not hasattr(features, '__iter__'):
@@ -288,11 +302,12 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
                 'model_name'  : model_name,
                 'solver'      : solver,
                 'l1_penalty'  : L1_penalty,
+                'feature_rescaling'  : feature_rescaling,
                 'l2_penalty'  : L2_penalty})
 
     # Pre-training
     print "Starting pre-training..."
-    ret = _graphlab.toolkits.main.run("regression_train_init", opts)
+    ret = _graphlab.toolkits.main.run("supervised_learning_train_init", opts)
     opts.update(ret)
 
     # Switching point for vw
@@ -332,7 +347,7 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
 
         # Print some output for ipython notebook users
         print "Starting training..."
-        ret = _graphlab.toolkits.main.run("regression_train", opts,
+        ret = _graphlab.toolkits.main.run("supervised_learning_train", opts,
                 verbose)
 
         model_proxy = ret['model']
@@ -342,14 +357,134 @@ def create(dataset, target, features=None, L2_penalty=1e-2, L1_penalty=0.0,
     return model
 
 
-class LinearRegressionModel(RegressionModel):
+class LinearRegressionModel(SupervisedLearningModel):
     """
-    Linear regression is an approach for modeling a scalar target :math:`y` as a
-    linear function of one or more explanatory variables denoted
-    :math:`X`.
+    Linear regression is an approach for modeling a scalar target :math:`y` as
+    a linear function of one or more explanatory variables denoted :math:`X`.
+    An instance of this model can be created using
+    :func:`graphlab.linear_regression.create`. Do not construct the model
+    directly.
+
+
+    **Examples**
+
+
+    .. sourcecode:: python
+
+        # Load the data (From an S3 bucket)
+        >>> import graphlab as gl
+        >>> data =  gl.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        # Make a linear regression model
+        >>> model = gl.linear_regression.create(data, target='expensive', features=['bath', 'bedroom', 'size'])
+
+        # Here, all columns (other than the target) are used as features.
+        >>> model = gl.linear_regression.create(data, target='expensive')
+
+        # Extract the coefficients
+        >>> coefficients = model['coefficients']     # an SFrame
+        
+        # Make predictions (as margins, probability, or class)
+        >>> predictions = model.predict(data)    # Predicts 0/1
+        >>> predictions = model.predict(data, output_type='probability')
+        >>> predictions = model.predict(data, output_type='margin')       
+
+        # Evaluate the model 
+        >>> results = m.evaluate(data)               # a dictionary
+    
+    **Matematical Details**
+    
+    Given a set of features :math:`x_i`, and a label :math:`y_i`,
+    linear regression interprets the probability that the label is in one class
+    as a linear function of a linear combination of the features.
+
+        .. math::
+          f_i(\\theta) =  \\theta^T x + \epsilon_i
+
+    where :math:`\epsilon_i` is noise.  An intercept term is added by appending
+    a column of 1's to the features.  Regularization is often required to
+    prevent overfitting by penalizing models with extreme parameter values. The
+    linear regression module supports L1 and L2 regularization, which are
+    added to the loss function.
+    
+    The composite objective being optimized for is the following:
+
+        .. math::
+           \min_{\\theta} \sum_{i = 1}^{n} (\\theta^Tx - y_i)^2 + \lambda_1 ||\\theta||_1 + \lambda_2 ||\\theta||^{2}_{2} 
+
+    where :math:`\lambda_1` is the ``L1_penalty`` and :math:`\lambda_2` is the
+    ``L2_penalty``.
+
+    **Model creation options guide**
+
+
+    solver: string, default = 'auto'
+
+        The optimization used to train the model is a carefully engineered
+        collection of methods that are automatically picked based on the input
+        data. The ``newton`` method  works best for datasets with plenty of
+        examples and few features (long datasets). Limited memory BFGS
+        (``lbfgs``) is a robust solver for wide datasets (i.e datasets with
+        many coefficients).  ``fista'' is the default solver for L1-regularized
+        linear regression. Gradient-descent (GD) is another well tuned methods
+        that can work really well on L1-regularized problems.  The solvers are
+        all automatically tuned and the default options should function well.
+        See the solver options guide for setting additional parameters for each
+        of the solvers.
+    
+
+    feature_rescaling: boolean, default = true
+
+        Feature rescaling is an important pre-processing step that ensures
+        that all features are on the same scale. An L2-norm rescaling is
+        performed to make sure that all features are of the same norm. Categorical
+        features are also rescaled by rescaling the dummy variables that 
+        are used to represent them. The coefficients are returned in original
+        scale of the problem.
+
+
+    **Solver Options Guide**
+
+    Most solvers are all automatically tuned and the default options should
+    function well. Changing the default settings may help in some cases.
+    
+
+    step_size : double, default = 1.0 (fista only)
+
+        The starting step size to use for the ``fista'' and ``gd'' solvers. The
+        default is set to 1.0, this is an aggressive setting. If the first
+        iteration is taking a considerable amount of time, then reduce this
+        step size to speed up model training. If the first iteration is very
+        fast, then consider increasing this step size to a more aggressive
+        value.
+
+    max_iterations : integer, default = 10.
+
+        The maximum number of passes through the data allowed. Increasing his 
+        can result in more accurate training time. If the training accuracy 
+        is low and the ``Grad-Norm`` in the display is large, then the model 
+        has not trained completely.
+
+    convergence_threshold : double, default = 1e-2.
+
+        Convergence is tested using variation in the training objective.  The
+        variation in the training objective is calculated by dividing the
+        difference between the objective values between two step. 
+    
+    lbfgs_memory_level: integer, default = 10.
+
+        The L-BFGS algorithm keeps track of gradient information from the
+        previous ``lbfgs_memory_level`` iterations. The storage requirement for
+        each of the gradients is the number of ``coefficients`` in the problem.
+        Increasing the ``lbfgs_memory_level`` can help improve the quality of
+        the model trained at the cost of more memory. For problems with fewer
+        coefficients (less than a few million), increasing the
+        ``lbfgs_memory_level`` is recommended for better quality of the
+        solution. Setting this to more than ``max_iterations`` has no more
+        benefits as setting it to the same as the ``max_iterations``.
+
+
 
-    An instance of this model can be created using :func:`graphlab.linear_regression.create`.
-    Do NOT construct the model directly.
     """
     def __init__(self, model_proxy, use_vw=False):
         '''__init__(self)'''
@@ -526,18 +661,18 @@ class LinearRegressionModel(RegressionModel):
                         "list_fields() for a list of keys that can be " \
                         "queried." % field
             return _vw.VWModel(self.__proxy__).get(vw_option_map[field])
-
-        opts = {'model': self.__proxy__,
-                'model_name': self.__name__,
-                'field': field}
-        response = _graphlab.toolkits.main.run('regression_get_value',
-            opts)
-
-        # Coefficients returns a unity SFrame. Cast to an SFrame.
-        if field == 'coefficients':
-            return _SFrame(None, _proxy=response['value'])
         else:
-            return response['value']
+            opts = {'model': self.__proxy__,
+                    'model_name': self.__name__,
+                    'field': field}
+            response = _graphlab.toolkits.main.run('supervised_learning_get_value',
+                opts)
+
+            # Coefficients returns a unity SFrame. Cast to an SFrame.
+            if field == 'coefficients':
+                return _SFrame(None, _proxy=response['value'])
+            else:
+                return response['value']
 
     def summary(self):
         """
@@ -548,7 +683,36 @@ class LinearRegressionModel(RegressionModel):
         the queryable fields.
         """
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.summary')
-        return super(LinearRegressionModel, self).summary()
+        solver = self.get('solver')
+        if solver == 'vw':
+            _vw.VWModel(self.__proxy__).summary()
+            return
+        else:
+            coefs = self.get('coefficients')
+            top_coefs = coefs.topk('Coefficient', k=6)
+            top_coefs = top_coefs[top_coefs['Coefficient'] > 0]
+
+            bottom_coefs = coefs.topk('Coefficient', k=5, reverse=True)
+            bottom_coefs = bottom_coefs[bottom_coefs['Coefficient'] < 0]
+
+            print ""
+            print "                    Model summary                       "
+            print "--------------------------------------------------------"
+            print self.__repr__()
+
+            print "             Strongest positive coefficients            "
+            print "--------------------------------------------------------"
+            if len(top_coefs) > 0:
+                print _SFrame(top_coefs)
+            else:
+                print "[No positive coefficients]"
+
+            print "             Strongest negative coefficients            "
+            print "--------------------------------------------------------"
+            if len(bottom_coefs) > 0:
+                print _SFrame(bottom_coefs)
+            else:
+                print "[No negative coefficients]"
 
     def get_default_options(self):
         """
@@ -561,9 +725,12 @@ class LinearRegressionModel(RegressionModel):
         """
 
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.get_default_options')
-        return super(LinearRegressionModel, self).get_default_options()
+        if self.get('solver') == 'vw':
+            return {'max_iterations': 10, 'step_size': 1.0}
+        else:
+            return super(LinearRegressionModel, self).get_default_options()
 
-    def get_options(self):
+    def get_current_options(self):
         """
         A dictionary describing the options requested during training.
 
@@ -575,9 +742,14 @@ class LinearRegressionModel(RegressionModel):
         """
 
         _mt._get_metric_tracker().track('toolkit.regression.linear_regression.get_options')
-        return super(LinearRegressionModel, self).get_options()
+        if self.get('solver') == 'vw':
+            max_iterations = _vw.VWModel(self.__proxy__).get('max_iterations')
+            step_size = _vw.VWModel(self.__proxy__).get('step_size')
+            return {'max_iterations': max_iterations, 'step_size': step_size}
+        else:
+            return super(LinearRegressionModel, self).get_current_options()
 
-    def predict(self, dataset):
+    def predict(self, dataset, missing_value_action='impute'):
         """
         Return target value predictions for ``dataset``, using the trained
         linear regression model. This method can be used to get fitted values
@@ -590,6 +762,18 @@ class LinearRegressionModel(RegressionModel):
             names as the features used for model training, but does not require
             a target column. Additional columns are ignored.
 
+        missing_value_action: str, optional
+            Action to perform when missing values are encountered. This can be 
+            one of:
+
+            - 'impute': Proceed with evaluation by filling in the missing
+                        values with the mean of the training data. Missing
+                        values are also imputed if an entire column of data is
+                        missing during evaluation.
+            - 'error' : Do not proceed with prediction and terminate with 
+                        an error message.
+
+
         Returns
         -------
         out : SArray
@@ -598,9 +782,26 @@ class LinearRegressionModel(RegressionModel):
 
         _mt._get_metric_tracker().track(
             'toolkit.regression.linear_regression.predict')
-        return super(LinearRegressionModel, self).predict(dataset)
 
-    def evaluate(self, dataset):
+        # Special handling of vw model
+        if self.get('solver') == 'vw':
+            m = _vw.VWModel(self.__proxy__)
+            predictions = m.predict(dataset)
+            return predictions
+        else:
+            # Get metadata from C++ object
+            opts = {'model': self.__proxy__,
+                    'model_name': self.__name__,
+                    'dataset': dataset,
+                    'missing_value_action': missing_value_action,
+                    'output_type': ""}
+
+            response = _graphlab.toolkits.main.run('supervised_learning_predict_init', opts)
+            opts.update(response)
+            response = _graphlab.toolkits.main.run('supervised_learning_predict', opts)
+            return _SArray(None, _proxy=response['predicted'])
+
+    def evaluate(self, dataset, missing_value_action = 'impute'):
         r"""Evaluate the model by making target value predictions and comparing
         to actual values.
 
@@ -627,6 +828,17 @@ class LinearRegressionModel(RegressionModel):
             Dataset of new observations. Must include columns with the same
             names as the target and features used for model training. Additional
             columns are ignored.
+        
+        missing_value_action: str, optional
+            Action to perform when missing values are encountered. This can be 
+            one of:
+
+            - 'impute': Proceed with evaluation by filling in the missing
+                        values with the mean of the training data. Missing
+                        values are also imputed if an entire column of data is
+                        missing during evaluation.
+            - 'error' : Do not proceed with evaluation and terminate with 
+                        an error message.
 
         Returns
         -------
@@ -639,8 +851,24 @@ class LinearRegressionModel(RegressionModel):
           <http://en.wikipedia.org/wiki/Root-mean-square_deviation>`_
         """
 
-        _mt._get_metric_tracker().track('toolkit.regression.linear_regression.evaluate')
-        return super(LinearRegressionModel, self).evaluate(dataset)
+        _mt._get_metric_tracker().track(
+            'toolkit.regression.linear_regression.evaluate')
+        if self.get('solver') == 'vw':
+            m = _vw.VWModel(self.__proxy__)
+            evaluation_results = m.evaluate(dataset)
+            return evaluation_results
+
+        opts = {'model': self.__proxy__,
+                'dataset': dataset,
+                'missing_value_action': missing_value_action,
+                'model_name': self.__name__}
+
+        response = _graphlab.toolkits.main.run('supervised_learning_evaluate_init', opts)
+        opts.update(response)
+        response = _graphlab.toolkits.main.run('supervised_learning_evaluate', opts)
+        opts.update(response)
+        return _graphlab.toolkits.main.run('supervised_learning_get_evaluate_stats', opts)
+
 
     def training_stats(self):
         """
@@ -712,6 +940,6 @@ class LinearRegressionModel(RegressionModel):
         else:
             opts = {'model': self.__proxy__,
                     'model_name': self.__name__}
-            response = _graphlab.toolkits.main.run('regression_list_keys', opts)
+            response = _graphlab.toolkits.main.run('supervised_learning_list_keys', opts)
             return sorted(response.keys())
 
