@@ -3,11 +3,11 @@ Methods for creating and using a logistic regression model.
 """
 import graphlab.connect as _mt
 import graphlab as _graphlab
-from graphlab.toolkits.regression.regression import RegressionModel
+from graphlab.toolkits.supervised_learning import SupervisedLearningModel
 from graphlab.data_structures.sframe import SArray as _SArray
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from graphlab import vowpal_wabbit as _vw
-from pandas import DataFrame as _DataFrame
+from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
 import math
 from graphlab.toolkits.main import ToolkitError
 
@@ -17,9 +17,13 @@ DEFAULT_SOLVER_OPTIONS = {
 'mini_batch_size': 1,
 'max_iterations': 10}
 
+DEFAULT_HYPER_PARAMETER_RANGE = {
+    'L1_penalty' : [0.0, 0.001, 0.01, 0.1, 1.0, 10.0, 100.0],
+    'L2_penalty' : [0.0, 0.001, 0.01, 0.1, 1.0, 10.0, 100.0]
+}
 
 def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
-           solver='auto', solver_options=None, verbose=True):
+    solver='auto', feature_rescaling=True, solver_options=None, verbose=True):
     """
     Create a :class:`~graphlab.logistic_regression.LogisticRegressionModel` to
     predict the class of a binary target variable based on a model of class
@@ -32,7 +36,7 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
 
     Parameters
     ----------
-    dataset : pandas.DataFrame/SFrame
+    dataset : SFrame
         Dataset for training the model.
 
     target : string
@@ -43,16 +47,25 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
         Names of the columns containing features. 'None' (the default) indicates
         that all columns except the target variable should be used as features.
 
+        The features are columns in the input SFrame that can be of the
+        following types:
+
         - *Numeric*: values of numeric type integer or float.
 
         - *Categorical*: values of type string.
 
-        - *List*: list of numeric (integer or float) values. Each list element
+        - *Array*: list of numeric (integer or float) values. Each list element
           is treated as a separate feature in the model.
 
-        - *Dictionary*: key-value pairs with numeric (integer or float) values.
-          Each key of a dictionary is treated as a categorical variable in the
-          model.
+        - *Dictionary*: key-value pairs with numeric (integer or float) values
+          Each key of a dictionary is treated as a separate feature and the
+          value in the dictionary corresponds to the value of the feature. 
+          Dictionaries are ideal for representing sparse data.
+          
+        Columns of type *list* are not supported. Convert such feature
+        columns to type array if all entries in the list are of numeric
+        types. If the lists contain data of mixed types, separate
+        them out into different columns.
 
     L2_penalty : float, optional
         Weight on L2 regularization of the model. The larger this weight, the
@@ -85,6 +98,11 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
         iteratively re-weighted least squares algorithm. If the L1_penalty is
         greater than 0, use either the 'fista' or 'vw' solver.
 
+    feature_rescaling: boolean, optional
+        Feature rescaling is the process of scaling individual features to
+        have unit L2-norm. This process is particularly useful when features
+        vary widely in their ranges.  (Not applicable when solver = 'vw')
+
     solver_options : dict, optional
         Solver options. The options and their default values are as follows:
 
@@ -97,7 +115,7 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
         +-----------------------+---------+-----------------------------------------+
         | max_iterations        |     10  | Max number of solver iterations         |
         +-----------------------+---------+-----------------------------------------+
-        | step_size             |    1.0  | Initial solver step size                |
+        | step_size             |    1.0  | Initial solver step size (fista only)   |
         +-----------------------+---------+-----------------------------------------+
 
     verbose : bool, optional
@@ -116,27 +134,21 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
       variables, while the first category encountered in the data is used as the
       baseline.
 
-    - For prediction and evaluation of logistic regression models with
-      categorical variables, test datasets cannot contain categories that were
-      not present during train time.
-
     - For prediction and evaluation of logistic regression models with sparse
       dictionary inputs, new keys/columns that were not seen during training
       are silently ignored.
 
-    - L2 and L1 regularization typically work best if the features are
-      standardized before model training by subtracting the mean of column and
-      dividing by the standard deviation. The regression model does *not* do
-      this automatically; use SFrame and SArray methods to transform the data
-      prior to training.
-
     - If the 'vw' solver is used, the model returns only predictions; it does
       not allow inspection of estimated effect sizes or standard errors.
 
-    - Any 'None' values in the data will result in an error being thrown.
+    - During model creation, 'None' values in the data will result in an error
+      being thrown.
 
     - A constant term is automatically added for the model intercept. This term
       is not regularized.
+    
+    - For solver='vw', the "step_size" option is the "learning_rate" parameter
+      in the  :class:`~graphlab.vowpal_wabbit.VWModel`. 
 
     Examples
     --------
@@ -239,13 +251,14 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
       <https://github.com/JohnLangford/vowpal_wabbit/wiki>`_
     """
 
+    print "Preparing the data..."
     model_name = "regression_logistic_regression"
     _mt._get_metric_tracker().track(
         'toolkit.regression.logistic_regression.create')
 
-
     ## Check data matrix type and convert to SFrame
-    if not isinstance(dataset, (_DataFrame, _SFrame)):
+    if not (isinstance(dataset, _SFrame) or (_HAS_PANDAS and
+      isinstance(dataset, _pandas.DataFrame))):
         raise TypeError('Dataset input must be an SFrame.')
 
     if type(dataset) != _SFrame:
@@ -270,7 +283,7 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
 
     # Check that target values are all +1 or 0
     zero_or_one = (sf_target[target] == 0) + (sf_target[target] == 1)
-    if not all(zero_or_one):
+    if not zero_or_one.all():
         raise ValueError("The target column for logistic regression" + \
             " must contain only 0 and 1 values.")
 
@@ -295,12 +308,13 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
                 'features'    : sf_features,
                 'model_name'  : model_name,
                 'solver'      : solver,
+                'feature_rescaling'  : feature_rescaling,
                 'l2_penalty'  : L2_penalty,
                 'l1_penalty'  : L1_penalty})
 
 
     # Model checking
-    ret = _graphlab.toolkits.main.run("regression_train_init", opts)
+    ret = _graphlab.toolkits.main.run("supervised_learning_train_init", opts)
     opts.update(ret)
     model = LogisticRegressionModel(ret['model'])
 
@@ -342,7 +356,7 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
     else:  # all other solvers beside VW
 
         try:
-            ret = _graphlab.toolkits.main.run("regression_train", opts, verbose)
+            ret = _graphlab.toolkits.main.run("supervised_learning_train", opts, verbose)
         except:
             raise ValueError("Model failed to train")
         else:
@@ -353,13 +367,141 @@ def create(dataset, target, features=None, L2_penalty=0.01, L1_penalty=0.0,
     return model
 
 
-class LogisticRegressionModel(RegressionModel):
+class LogisticRegressionModel(SupervisedLearningModel):
     """
-    Logistic regression models a binary target variable as a function of several
-    feature variables.
+    Logistic regression models a binary target variable as a function of
+    several feature variables. An instance of this model can be created using
+    :func:`graphlab.logistic_regression.create`.  Do not construct the model
+    directly. A detailed list of options are available in the create function.
+
+
+    The :class:`~graphlab.logistic_regression.LogisticRegressionModel` uses
+    a binary target variable :math:`y` instead of a scalar. For each
+    observation, the probability that :math:`y=1` (instead of 0) is modeled as
+    the logistic function of a linear combination of the feature values.
+    Logistic regression predictions may take the form of the linear combination
+    (also known as the margin), the probability that :math:`y = 1`, or a binary
+    class value, which is obtained by thresholding the probability at 0.5. The
+    :class:`~graphlab.evaluation` toolkit gives you more flexible evaluation metrics.
+    
+
+    **Examples**
+
+
+    .. sourcecode:: python
+
+        # Load the data (From an S3 bucket)
+        >>> import graphlab as gl
+        >>> data =  gl.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/regression/houses.csv')
+
+        # Make sure the target is binary 0/1
+        >>> data['expensive'] = data['price'] > 30000
+        
+        # Make a logistic regression model
+        >>> model = gl.logistic_regression.create(data, target='expensive', features=['bath', 'bedroom', 'size'])
+
+        # Here, all columns (other than the target) are used as features.
+        >>> model = gl.logistic_regression.create(data, target='expensive')
+
+        # Extract the coefficients
+        >>> coefficients = model['coefficients']     # an SFrame
+        
+        # Make predictions (as margins, probability, or class)
+        >>> predictions = model.predict(data)    # Predicts 0/1
+        >>> predictions = model.predict(data, output_type='probability')
+        >>> predictions = model.predict(data, output_type='margin')       
+
+        # Evaluate the model 
+        >>> results = m.evaluate(data)               # a dictionary
+    
+    **Matematical Details**
+    
+    Given a set of features :math:`x_i`, and a label :math:`y_i \in \{0,1\}`,
+    logistic regression interprets the probability that the label is in one class
+    as a logistic function of a linear combination of the features.
+
+        .. math::
+          f_i(\\theta) =  p(y_i = 1 | x) = \\frac{1}{1 + \exp(-\\theta^T x)}
+
+    An intercept term is added by appending a column of 1's to the features.
+    Regularization is often required to prevent over fitting by penalizing
+    models with extreme parameter values. The logistic regression module
+    supports L1 and L2 regularization, which are added to the loss function.
+    
+    The composite objective being optimized for is the following:
+
+        .. math::
+           \min_{\\theta} \sum_{i = 1}^{n} f_i(\\theta) + \lambda_1 ||\\theta||_1 + \lambda_2 ||\\theta||^{2}_{2} 
+
+    where :math:`\lambda_1` is the ``L1_penalty`` and :math:`\lambda_2` is the
+    ``L2_penalty``.
+
+    **Model creation options guide**
+
+
+    solver: string, default = 'auto'
+
+        The optimization used to train the model is a carefully engineered
+        collection of methods that are automatically picked based on the input
+        data. The ``newton`` method  works best for datasets with plenty
+        of examples and few features (long datasets). Limited memory BFGS
+        (``lbfgs``) is a robust solver for wide datasets (i.e datasets with many
+        coefficients).  ``fista'' is the default solver for L1-regularized logistic
+        regression.  The solvers are all automatically tuned and the default
+        options should function well. See the solver options guide for setting
+        additional parameters for each of the solvers.
+    
+
+    feature_rescaling: string, default = true
+
+        Feature rescaling is an important pre-processing step that ensures
+        that all features are on the same scale. An L2-norm rescaling is
+        performed to make sure that all features are of the same norm. Categorical
+        features are also rescaled by rescaling the dummy variables that 
+        are used to represent them. The coefficients are returned in original
+        scale of the problem.
+
+
+    **Solver Options Guide**
+
+    Most solvers are all automatically tuned and the default options should
+    function well. Changing the default settings may help in some cases.
+    
+
+    step_size : double, default = 1.0 (fista only)
+
+        The starting step size to use for the ``fista'' solver. The default is set to 
+        1.0, this is an aggressive setting. If the first iteration is taking 
+        a considerable amount of time, then reduce this step size to speed
+        up model training. If the first iteration is very fast, then consider
+        increasing this step size to a more aggressive value.
+
+    max_iterations : integer, default = 10.
+
+        The maximum number of passes through the data allowed. Increasing his 
+        can result in more accurate training time. If the training accuracy 
+        is low and the ``Grad-Norm`` in the display is large, then the model 
+        has not trained completely.
+
+    convergence_threshold : double, default = 1e-2.
+
+        Convergence is tested using variation in the training objective.  The
+        variation in the training objective is calculated by dividing the
+        difference between the objective values between two step. 
+    
+    lbfgs_memory_level: integer, default = 10.
+
+        The L-BFGS algorithm keeps track of gradient information from the
+        previous ``lbfgs_memory_level`` iterations. The storage requirement for
+        each of the gradients is the number of ``coefficients`` in the problem.
+        Increasing the ``lbfgs_memory_level`` can help improve the quality of the
+        model trained at the cost of more memory. For problems with fewer
+        coefficients (less than a few million), increasing the
+        ``lbfgs_memory_level`` is recommended for better quality of the
+        solution. Setting this to more than ``max_iterations`` has no more
+        benefits as setting it to the same as the ``max_iterations``.
+
 
-    An instance of this model can be created using :func:`graphlab.logistic_regression.create`.
-    Do NOT construct the model directly.
     """
     def __init__(self, model_proxy, use_vw=False):
         '''__init__(self)'''
@@ -530,7 +672,7 @@ class LogisticRegressionModel(RegressionModel):
         else:
             opts = {'model': self.__proxy__, 'model_name': self.__name__,
                     'field': field}
-            response = _graphlab.toolkits.main.run('regression_get_value', opts)
+            response = _graphlab.toolkits.main.run('supervised_learning_get_value', opts)
 
             # Coefficients returns a unity SFrame. Cast to an SFrame.
             if field == 'coefficients':
@@ -548,7 +690,36 @@ class LogisticRegressionModel(RegressionModel):
         """
         _mt._get_metric_tracker().track(
             'toolkit.regression.logistic_regression.summary')
-        super(LogisticRegressionModel, self).summary()
+        solver = self.get('solver')
+        if solver == 'vw':
+            _vw.VWModel(self.__proxy__).summary()
+            return
+        else:
+            coefs = self.get('coefficients')
+            top_coefs = coefs.topk('Coefficient', k=6)
+            top_coefs = top_coefs[top_coefs['Coefficient'] > 0]
+
+            bottom_coefs = coefs.topk('Coefficient', k=5, reverse=True)
+            bottom_coefs = bottom_coefs[bottom_coefs['Coefficient'] < 0]
+
+            print ""
+            print "                    Model summary                       "
+            print "--------------------------------------------------------"
+            print self.__repr__()
+
+            print "             Strongest positive coefficients            "
+            print "--------------------------------------------------------"
+            if len(top_coefs) > 0:
+                print _SFrame(top_coefs)
+            else:
+                print "[No positive coefficients]"
+
+            print "             Strongest negative coefficients            "
+            print "--------------------------------------------------------"
+            if len(bottom_coefs) > 0:
+                print _SFrame(bottom_coefs)
+            else:
+                print "[No negative coefficients]"
 
     def get_default_options(self):
         """
@@ -559,11 +730,13 @@ class LogisticRegressionModel(RegressionModel):
         out : dict
             Dictionary with model default options.
         """
-        _mt._get_metric_tracker().track(
-           'toolkit.regression.logistic_regression.get_default_options')
-        return super(LogisticRegressionModel, self).get_default_options()
+        _mt._get_metric_tracker().track('toolkit.regression.logistic_regression.get_default_options')
+        if self.get('solver') == 'vw':
+            return {'max_iterations': 10, 'step_size': 1.0}
+        else:
+            return super(LogisticRegressionModel, self).get_default_options()
 
-    def get_options(self):
+    def get_current_options(self):
         """
         Return a dictionary with the options used to define and train the model.
 
@@ -572,11 +745,15 @@ class LogisticRegressionModel(RegressionModel):
         out : dict
             Dictionary with options used to define and train the model.
         """
-        _mt.get_metric_tracker().track(
-           'toolkit.regression.logistic_regression.get_options')
-        return super(LogisticRegressionModel, self).get_options()
+        if self.get('solver') == 'vw':
+            max_iterations = _vw.VWModel(self.__proxy__).get('max_iterations')
+            step_size = _vw.VWModel(self.__proxy__).get('step_size')
+            return {'max_iterations': max_iterations, 'step_size': step_size}
+        else:
+            return super(LogisticRegressionModel, self).get_current_options()
 
-    def predict(self, dataset, output_type='probability'):
+    def predict(self, dataset, output_type='probability',
+        missing_value_action='impute'):
         """
         Return predictions for ``dataset``, using the trained logistic
         regression model. Predictions can be generated as class labels (0 or 1),
@@ -588,7 +765,9 @@ class LogisticRegressionModel(RegressionModel):
         linear predictor---is the inner product of the example and the model
         coefficients. The probability is obtained by passing the margin through
         the logistic function. Predicted classes are obtained by thresholding
-        the predicted probabilities at 0.5.
+        the predicted probabilities at 0.5. If you would like to threshold
+        predictions at a different probability level, you can use the 
+        Graphlab evaluation toolkit.
 
         Parameters
         ----------
@@ -599,6 +778,17 @@ class LogisticRegressionModel(RegressionModel):
 
         output_type : {'probability', 'margin', 'class'}, optional
             Form of the predictions.
+        
+        missing_value_action: str, optional
+            Action to perform when missing values are encountered. This can be 
+            one of:
+
+            - 'impute': Proceed with evaluation by filling in the missing
+                        values with the mean of the training data. Missing
+                        values are also imputed if an entire column of data is
+                        missing during evaluation.
+            - 'error' : Do not proceed with prediction and terminate with 
+                        an error message.
 
         Returns
         -------
@@ -613,7 +803,7 @@ class LogisticRegressionModel(RegressionModel):
             raise ValueError("Output type '{}' is not supported.".format(output_type) + \
                              " Please select 'probability', 'margin', or 'class'.")
 
-        if type(dataset) == _DataFrame:
+        if _HAS_PANDAS and type(dataset) == _pandas.DataFrame:
             dataset = _SFrame(dataset)
 
         # special handling of vw model
@@ -642,20 +832,21 @@ class LogisticRegressionModel(RegressionModel):
                     predictions = predictions.apply(lambda x: x >= 0.5)
                     return predictions
 
+        else:
+            # Get metadata from C++ object
+            opts = {'model': self.__proxy__,
+                    'model_name': self.__name__,
+                    'dataset': dataset,
+                    'missing_value_action': missing_value_action,
+                    'output_type': output_type}
 
-        # Get metadata from C++ object
-        opts = {'model': self.__proxy__,
-                'model_name': self.__name__,
-                'test_data': dataset,
-                'output_type': output_type}
+            ## Compute the predictions
+            init_opts = _graphlab.toolkits.main.run('supervised_learning_predict_init', opts)
+            opts.update(init_opts)
+            target = _graphlab.toolkits.main.run('supervised_learning_predict', opts)
+            return _SArray(None, _proxy=target['predicted'])
 
-        ## Compute the predictions
-        init_opts = _graphlab.toolkits.main.run('regression_predict_init', opts)
-        opts.update(init_opts)
-        target = _graphlab.toolkits.main.run('regression_predict', opts)
-        return _SArray(None, _proxy=target['predicted'])
-
-    def evaluate(self, dataset):
+    def evaluate(self, dataset, missing_value_action = 'impute'):
         """
         Evaluate the model by making predictions of target values and comparing
         these to actual values.
@@ -672,6 +863,17 @@ class LogisticRegressionModel(RegressionModel):
             names as the target and features used for model training. Additional
             columns are ignored.
 
+        missing_value_action: str, optional
+            Action to perform when missing values are encountered. This can be 
+            one of:
+
+            - 'impute': Proceed with evaluation by filling in the missing
+                        values with the mean of the training data. Missing
+                        values are also imputed if an entire column of data is
+                        missing during evaluation.
+            - 'error' : Do not proceed with evaluation and terminate with 
+                        an error message.
+
         Returns
         -------
         out : dict
@@ -713,13 +915,25 @@ class LogisticRegressionModel(RegressionModel):
 
             evaluation_results = m.evaluate(dataset_copy)
             return evaluation_results
+        else:
+            opts = {'model': self.__proxy__,
+                    'dataset': dataset,
+                    'missing_value_action': missing_value_action,
+                    'model_name': self.__name__}
+
+            response = _graphlab.toolkits.main.run('supervised_learning_evaluate_init', opts)
+            opts.update(response)
+            response = _graphlab.toolkits.main.run("supervised_learning_evaluate", opts)
+            opts.update(response)
+            results =  _graphlab.toolkits.main.run("supervised_learning_get_evaluate_stats",
+                    opts)
+            keys = ['true_positive', 'true_negative', 'false_positive',
+                    'false_negative']
+            confusion_matrix = {k: results[k] for k in keys}
+            accuracy = results['accuracy']
+            return {'accuracy': accuracy,
+                    'confusion_table': confusion_matrix}
 
-        targets = dataset[target_column]
-        predictions = self.predict(dataset)
-        accuracy = _graphlab.evaluation.accuracy(targets, predictions)
-        confusion_matrix = _graphlab.evaluation.confusion_matrix(targets, predictions)
-        return {'accuracy': accuracy,
-                'confusion_table': confusion_matrix}
 
     def training_stats(self):
         """
@@ -768,5 +982,5 @@ class LogisticRegressionModel(RegressionModel):
         else:
             opts = {'model': self.__proxy__,
                     'model_name': self.__name__}
-            response = _graphlab.toolkits.main.run('regression_list_keys', opts)
+            response = _graphlab.toolkits.main.run('supervised_learning_list_keys', opts)
             return sorted(response.keys())
