@@ -33,7 +33,7 @@ import graphlab.connect as _mt
 from graphlab.toolkits.model import Model
 from graphlab.data_structures.sframe import SFrame as _SFrame
 from graphlab.data_structures.sarray import SArray as _SArray
-from pandas import DataFrame as _DataFrame
+from graphlab.deps import pandas as _pandas, HAS_PANDAS as _HAS_PANDAS
 import time
 
 class VWModel(Model):
@@ -354,7 +354,7 @@ def create(dataset, target_column,
 
     _mt._get_metric_tracker().track('toolkit.vowpal_wabbit.create')
 
-    if not isinstance(dataset, (_DataFrame, _SFrame)):
+    if not (isinstance(dataset, _SFrame) or _HAS_PANDAS and isinstance(dataset, _pandas.DataFrame)):
         raise TypeError('dataset input must be a pandas.DataFrame or SFrame')
 
     if type(dataset) != _SFrame:
