@@ -46,7 +46,6 @@ class _MetricsWorkerThread(threading.Thread):
     while True:
       try:
         metric = self.queue.get() # block until something received
-        self.logger.debug("GOT WORK! %s" % metric)
 
         if (metric['event_name'] == SHUTDOWN_MESSAGE):
           # shutting down, send total session time and then exit
@@ -59,7 +58,6 @@ class _MetricsWorkerThread(threading.Thread):
         self.tracker._track(metric['event_name'], metric['value'], metric['type'], metric['properties'], metric['meta'], metric['send_sys_info'])
                 
         self.queue.task_done()
-        self.logger.debug("logged '%s' successfully" % metric['event_name'])
 
       except Queue.Empty:
         # if queue is empty, wait try again
@@ -133,7 +131,6 @@ class MetricTracker:
     try:
       item = dict(event_name=event_name, value=value, type=type, properties=properties, meta=meta, send_sys_info=send_sys_info)
       self._queue.put_nowait(item) # don't wait if Queue is full, just silently ignore
-      self.logger.debug("Enqueued event: %s, current queue size: %d" % (event_name, self._queue.qsize()))
     except Queue.Full:
       if not self._thread or not self._thread.is_alive():
         self.logger.debug("Queue is full and background thread is no longer alive, trying to restart")
@@ -232,6 +229,23 @@ class MetricTracker:
     except:
       pass
 
+  @staticmethod
+  def _get_bucket_name_suffix(buckets, value):
+    """
+    Given a list of buckets and a value, generate a suffix for the bucket
+    name, corresponding to either one of the buckets given, or the largest
+    bucket with "+" appended.
+    """
+    suffix = None
+    for bucket in buckets:
+        if value <= bucket:
+            suffix = str(bucket)
+            break
+    # if we get here and suffix is None, value must be > the largest bucket
+    if suffix is None:
+        suffix = '%d+' % buckets[-1]
+    return suffix
+
   def _bucketize_mixpanel(self, event_name, value):
     """
     Take events that we would like to bucketize and bucketize them before sending to mixpanel
@@ -242,41 +256,23 @@ class MetricTracker:
     """
 
     if value == 1:
-      return event_name
+        return event_name
 
-    if event_name.endswith('col.size'):
-        col_buckets = [ 5, 10, 20 ]
-        suffix = None
-        for bucket in col_buckets:
-            if value <= bucket:
-                suffix = str(bucket)
-                break
-        if suffix is None and value > 20:
-            suffix = '20+'              
-        event_name = '%s.%s' % (event_name, suffix)
-    elif event_name.endswith('row.size'):
-        row_buckets = [ 100000, 1000000, 10000000, 100000000 ]
-        suffix = None
-        for bucket in row_buckets:
-            if value <= bucket:
-                suffix = str(bucket)
-                break
-        if suffix is None and value > 100000000:
-            suffix = '100000000+'
-        event_name = '%s.%s' % (event_name, suffix)
-    elif event_name.endswith('duration.secs'):
-        session_buckets = [ 300, 1800, 3600, 7200 ]
-        suffix = None
-        for bucket in session_buckets:
-            if value <= bucket:
-                suffix = str(bucket)
-                break
-        if suffix is None and value > 7200:
-            suffix = '7200+'
-        event_name = '%s.%s' % (event_name, suffix)
+    bucket_events = {
+        'col.size': [ 5, 10, 20 ],
+        'row.size': [ 100000, 1000000, 10000000, 100000000 ],
+        'duration.secs': [ 300, 1800, 3600, 7200 ],
+        'duration.ms': [ 10, 100, 1000, 10000, 100000 ]
+    }
 
+    for (event_suffix, buckets) in bucket_events.iteritems():
+        if event_name.endswith(event_suffix):
+            # if the suffix matches one we expect, bucketize using the buckets defined above
+            return '%s.%s' % (event_name, MetricTracker._get_bucket_name_suffix(buckets, value))
+
+    # if there was no suffix match, just use the original event name
     return event_name
-    
+
 
   def _set_sys_info(self):
     # Don't do this if system info has been set
